@@ -949,7 +949,7 @@ func TestSubErrorNegative(t *testing.T) {
 
 	// complex case: just checking the resource merge, values check is secondary
 	res1 = &Resource{Resources: map[string]Quantity{"a": 0, "b": 1}}
-	res2 := &Resource{Resources: map[string]Quantity{"a": 1, "c": 0, "d": -1}}
+	res2 := &Resource{Resources: map[string]Quantity{"a": 1, "c": 1, "d": -1}}
 	result, err = SubErrorNegative(res1, res2)
 	if err == nil {
 		t.Errorf("sub should have set error message and did not: %v", result)
@@ -961,6 +961,27 @@ func TestSubErrorNegative(t *testing.T) {
 	}
 }
 
+func TestEqualsOrEmpty(t *testing.T) {
+	var tests = []struct {
+		left, right *Resource
+		want        bool
+	}{
+		{nil, nil, true},
+		{nil, NewResourceFromMap(map[string]Quantity{"a": 0, "b": 1}), false},
+		{NewResourceFromMap(map[string]Quantity{"a": 0, "b": 1}), nil, false},
+		{nil, NewResource(), true},
+		{NewResource(), nil, true},
+		{NewResourceFromMap(map[string]Quantity{"a": 0, "b": 1}), NewResourceFromMap(map[string]Quantity{"a": 0, "b": 1}), true},
+		{NewResourceFromMap(map[string]Quantity{"a": 0, "c": 1}), NewResourceFromMap(map[string]Quantity{"a": 0, "d": 3}), false},
+	}
+
+	for _, tt := range tests {
+		if got := EqualsOrEmpty(tt.left, tt.right); got != tt.want {
+			t.Errorf("got %v, want %v", got, tt.want)
+		}
+	}
+}
+
 func TestFitIn(t *testing.T) {
 	// simple case (nil checks)
 	empty := NewResource()
