@@ -33,8 +33,8 @@ import (
 	"github.com/apache/incubator-yunikorn-core/pkg/common/security"
 	"github.com/apache/incubator-yunikorn-core/pkg/events"
 	"github.com/apache/incubator-yunikorn-core/pkg/handler"
-	"github.com/apache/incubator-yunikorn-core/pkg/interfaces"
 	"github.com/apache/incubator-yunikorn-core/pkg/log"
+	"github.com/apache/incubator-yunikorn-core/pkg/metrics"
 	"github.com/apache/incubator-yunikorn-core/pkg/rmproxy/rmevent"
 	"github.com/apache/incubator-yunikorn-scheduler-interface/lib/go/si"
 )
@@ -48,14 +48,15 @@ var (
 )
 
 const (
-	Soft string = "Soft"
-	Hard string = "Hard"
+	Soft                    string = "Soft"
+	Hard                    string = "Hard"
+	AppTagStateAwareDisable string = "application.stateaware.disable"
 )
 
 type Application struct {
 	ApplicationID  string
 	Partition      string
-	QueueName      string
+	QueuePath      string
 	SubmissionTime time.Time
 
 	// Private fields need protection
@@ -72,6 +73,7 @@ type Application struct {
 	placeholderAsk       *resources.Resource    // total placeholder request for the app (all task groups)
 	stateMachine         *fsm.FSM               // application state machine
 	stateTimer           *time.Timer            // timer for state time
+	startTimeout         time.Duration          // timeout for the application starting state
 	execTimeout          time.Duration          // execTimeout for the application run
 	placeholderTimer     *time.Timer            // placeholder replace timer
 	gangSchedulingStyle  string                 // gang scheduling style can be hard (after timeout we fail the application), or soft (after timeeout we schedule it as a normal application)
@@ -87,7 +89,7 @@ func NewApplication(siApp *si.AddApplicationRequest, ugi security.UserGroup, eve
 	app := &Application{
 		ApplicationID:        siApp.ApplicationID,
 		Partition:            siApp.PartitionName,
-		QueueName:            siApp.QueueName,
+		QueuePath:            siApp.QueueName,
 		SubmissionTime:       time.Now(),
 		tags:                 siApp.Tags,
 		pending:              resources.NewResource(),
@@ -105,12 +107,17 @@ func NewApplication(siApp *si.AddApplicationRequest, ugi security.UserGroup, eve
 	}
 	gangSchedStyle := siApp.GetGangSchedulingStyle()
 	if gangSchedStyle != Soft && gangSchedStyle != Hard {
-		log.Logger().Info("Unknown gang scheduling style, using hard style as default",
+		log.Logger().Info("Unknown gang scheduling style, using soft style as default",
 			zap.String("gang scheduling style", gangSchedStyle))
-		gangSchedStyle = Hard
+		gangSchedStyle = Soft
 	}
 	app.gangSchedulingStyle = gangSchedStyle
 	app.execTimeout = placeholderTimeout
+	if app.GetTag(AppTagStateAwareDisable) != "" {
+		app.startTimeout = 0 // transition immediately to Running
+	} else {
+		app.startTimeout = startingTimeout
+	}
 	app.user = ugi
 	app.rmEventHandler = eventHandler
 	app.rmID = rmID
@@ -121,8 +128,8 @@ func (sa *Application) String() string {
 	if sa == nil {
 		return "application is nil"
 	}
-	return fmt.Sprintf("ApplicationID: %s, Partition: %s, QueueName: %s, SubmissionTime: %x, State: %s",
-		sa.ApplicationID, sa.Partition, sa.QueueName, sa.SubmissionTime, sa.stateMachine.Current())
+	return fmt.Sprintf("ApplicationID: %s, Partition: %s, QueuePath: %s, SubmissionTime: %x, State: %s",
+		sa.ApplicationID, sa.Partition, sa.QueuePath, sa.SubmissionTime, sa.stateMachine.Current())
 }
 
 func (sa *Application) SetState(state string) {
@@ -550,7 +557,7 @@ func (sa *Application) RecoverAllocationAsk(ask *AllocationAsk) {
 	}
 }
 
-func (sa *Application) updateAskRepeat(allocKey string, delta int32) (*resources.Resource, error) {
+func (sa *Application) UpdateAskRepeat(allocKey string, delta int32) (*resources.Resource, error) {
 	sa.Lock()
 	defer sa.Unlock()
 	if ask := sa.requests[allocKey]; ask != nil {
@@ -750,7 +757,7 @@ func (sa *Application) getOutstandingRequests(headRoom *resources.Resource, tota
 
 // Try a regular allocation of the pending requests
 // This includes placeholders
-func (sa *Application) tryAllocate(headRoom *resources.Resource, nodeIterator func() interfaces.NodeIterator) *Allocation {
+func (sa *Application) tryAllocate(headRoom *resources.Resource, nodeIterator func() NodeIterator) *Allocation {
 	sa.Lock()
 	defer sa.Unlock()
 	// make sure the request are sorted
@@ -767,7 +774,7 @@ func (sa *Application) tryAllocate(headRoom *resources.Resource, nodeIterator fu
 		if !headRoom.FitInMaxUndef(request.AllocatedResource) {
 			// post scheduling events via the event plugin
 			if eventCache := events.GetEventCache(); eventCache != nil {
-				message := fmt.Sprintf("Application %s does not fit into %s queue", request.ApplicationID, sa.QueueName)
+				message := fmt.Sprintf("Application %s does not fit into %s queue", request.ApplicationID, sa.QueuePath)
 				if event, err := events.CreateRequestEventRecord(request.AllocationKey, request.ApplicationID, "InsufficientQueueResources", message); err != nil {
 					log.Logger().Warn("Event creation failed",
 						zap.String("event message", message),
@@ -792,7 +799,8 @@ func (sa *Application) tryAllocate(headRoom *resources.Resource, nodeIterator fu
 }
 
 // Try to replace a placeholder with a real allocation
-func (sa *Application) tryPlaceholderAllocate(nodeIterator func() interfaces.NodeIterator, getnode func(string) *Node) *Allocation {
+//nolint:funlen
+func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator, getnode func(string) *Node) *Allocation {
 	sa.Lock()
 	defer sa.Unlock()
 	// nothing to do if we have no placeholders allocated
@@ -869,11 +877,18 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() interfaces.Nod
 	// pick the first fit and try all nodes if that fails give up
 	if phFit != nil && reqFit != nil {
 		for iterator.HasNext() {
-			node, ok := iterator.Next().(*Node)
-			if !ok {
+			node := iterator.Next()
+			if node == nil {
 				log.Logger().Warn("Node iterator failed to return a node")
 				return nil
 			}
+			if err := node.IsValidFor(reqFit); err != nil {
+				log.Logger().Debug("skipping node for placeholder ask",
+					zap.String("allocationKey", reqFit.AllocationKey),
+					zap.String("node", node.NodeID),
+					zap.String("reason", err.Error()))
+				continue
+			}
 			if err := node.preAllocateCheck(reqFit.AllocatedResource, reservationKey(nil, sa, reqFit), false); err != nil {
 				continue
 			}
@@ -913,7 +928,7 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() interfaces.Nod
 }
 
 // Try a reserved allocation of an outstanding reservation
-func (sa *Application) tryReservedAllocate(headRoom *resources.Resource, nodeIterator func() interfaces.NodeIterator) *Allocation {
+func (sa *Application) tryReservedAllocate(headRoom *resources.Resource, nodeIterator func() NodeIterator) *Allocation {
 	sa.Lock()
 	defer sa.Unlock()
 	// process all outstanding reservations and pick the first one that fits
@@ -963,13 +978,20 @@ func (sa *Application) tryReservedAllocate(headRoom *resources.Resource, nodeIte
 
 // Try all the nodes for a reserved request that have not been tried yet.
 // This should never result in a reservation as the ask is already reserved
-func (sa *Application) tryNodesNoReserve(ask *AllocationAsk, iterator interfaces.NodeIterator, reservedNode string) *Allocation {
+func (sa *Application) tryNodesNoReserve(ask *AllocationAsk, iterator NodeIterator, reservedNode string) *Allocation {
 	for iterator.HasNext() {
-		node, ok := iterator.Next().(*Node)
-		if !ok {
+		node := iterator.Next()
+		if node == nil {
 			log.Logger().Warn("Node iterator failed to return a node")
 			return nil
 		}
+		if err := node.IsValidFor(ask); err != nil {
+			log.Logger().Debug("skipping node for reserved ask",
+				zap.String("allocationKey", ask.AllocationKey),
+				zap.String("node", node.NodeID),
+				zap.String("reason", err.Error()))
+			continue
+		}
 		// skip over the node if the resource does not fit the node or this is the reserved node.
 		if !node.FitInNode(ask.AllocatedResource) || node.NodeID == reservedNode {
 			continue
@@ -988,7 +1010,7 @@ func (sa *Application) tryNodesNoReserve(ask *AllocationAsk, iterator interfaces
 
 // Try all the nodes for a request. The result is an allocation or reservation of a node.
 // New allocations can only be reserved after a delay.
-func (sa *Application) tryNodes(ask *AllocationAsk, iterator interfaces.NodeIterator) *Allocation {
+func (sa *Application) tryNodes(ask *AllocationAsk, iterator NodeIterator) *Allocation {
 	var nodeToReserve *Node
 	scoreReserved := math.Inf(1)
 	// check if the ask is reserved or not
@@ -996,22 +1018,32 @@ func (sa *Application) tryNodes(ask *AllocationAsk, iterator interfaces.NodeIter
 	reservedAsks := sa.GetAskReservations(allocKey)
 	allowReserve := len(reservedAsks) < int(ask.pendingRepeatAsk)
 	for iterator.HasNext() {
-		node, ok := iterator.Next().(*Node)
-		if !ok {
+		node := iterator.Next()
+		if node == nil {
 			log.Logger().Warn("Node iterator failed to return a node")
 			return nil
 		}
+		// skip the node if the node is not valid for the ask
+		if err := node.IsValidFor(ask); err != nil {
+			log.Logger().Debug("skipping node for ask",
+				zap.String("allocationKey", ask.AllocationKey),
+				zap.String("node", node.NodeID),
+				zap.String("reason", err.Error()))
+			continue
+		}
 		// skip over the node if the resource does not fit the node at all.
 		if !node.FitInNode(ask.AllocatedResource) {
 			continue
 		}
+		tryNodeStart := time.Now()
 		alloc := sa.tryNode(node, ask)
 		// allocation worked so return
 		if alloc != nil {
+			metrics.GetSchedulerMetrics().ObserveTryNodeLatency(tryNodeStart)
 			// check if the node was reserved for this ask: if it is set the result and return
 			// NOTE: this is a safeguard as reserved nodes should never be part of the iterator
 			// but we have no locking
-			if _, ok = sa.reservations[reservationKey(node, nil, ask)]; ok {
+			if _, ok := sa.reservations[reservationKey(node, nil, ask)]; ok {
 				log.Logger().Debug("allocate found reserved ask during non reserved allocate",
 					zap.String("appID", sa.ApplicationID),
 					zap.String("nodeID", node.NodeID),
@@ -1109,10 +1141,10 @@ func (sa *Application) tryNode(node *Node, ask *AllocationAsk) *Allocation {
 	return nil
 }
 
-func (sa *Application) GetQueueName() string {
+func (sa *Application) GetQueuePath() string {
 	sa.RLock()
 	defer sa.RUnlock()
-	return sa.queue.QueuePath
+	return sa.QueuePath
 }
 
 func (sa *Application) GetQueue() *Queue {
@@ -1123,17 +1155,17 @@ func (sa *Application) GetQueue() *Queue {
 
 // Set the leaf queue the application runs in. The queue will be created when the app is added to the partition.
 // The queue name is set to what the placement rule returned.
-func (sa *Application) SetQueueName(queuePath string) {
+func (sa *Application) SetQueuePath(queuePath string) {
 	sa.Lock()
 	defer sa.Unlock()
-	sa.QueueName = queuePath
+	sa.QueuePath = queuePath
 }
 
 // Set the leaf queue the application runs in.
 func (sa *Application) SetQueue(queue *Queue) {
 	sa.Lock()
 	defer sa.Unlock()
-	sa.QueueName = queue.QueuePath
+	sa.QueuePath = queue.QueuePath
 	sa.queue = queue
 }
 
@@ -1416,3 +1448,10 @@ func (sa *Application) notifyRMAllocationAskReleased(rmID string, released []*Al
 	}
 	sa.rmEventHandler.HandleEvent(releaseEvent)
 }
+
+func (sa *Application) IsAllocationAssignedToApp(alloc *Allocation) bool {
+	sa.RLock()
+	defer sa.RUnlock()
+	_, ok := sa.allocations[alloc.UUID]
+	return ok
+}
