@@ -43,7 +43,7 @@ func TestNewApplication(t *testing.T) {
 	siApp := &si.AddApplicationRequest{}
 	app := NewApplication(siApp, user, nil, "")
 	assert.Equal(t, app.ApplicationID, "", "application ID should not be set was not set in SI")
-	assert.Equal(t, app.QueueName, "", "queue name should not be set was not set in SI")
+	assert.Equal(t, app.QueuePath, "", "queue name should not be set was not set in SI")
 	assert.Equal(t, app.Partition, "", "partition name should not be set was not set in SI")
 	assert.Equal(t, app.rmID, "", "RM ID should not be set was not passed in")
 	assert.Equal(t, app.rmEventHandler, handler.EventHandler(nil), "event handler should be nil")
@@ -75,7 +75,7 @@ func TestNewApplication(t *testing.T) {
 	}
 	app = NewApplication(siApp, user, &appEventHandler{}, "myRM")
 	assert.Equal(t, app.ApplicationID, "appID", "application ID should not be set to SI value")
-	assert.Equal(t, app.QueueName, "some.queue", "queue name should not be set to SI value")
+	assert.Equal(t, app.QueuePath, "some.queue", "queue name should not be set to SI value")
 	assert.Equal(t, app.Partition, "AnotherPartition", "partition name should be set to SI value")
 	if app.rmEventHandler == nil {
 		t.Fatal("non nil handler was not set in the new app")
@@ -256,11 +256,12 @@ func TestUpdateRepeat(t *testing.T) {
 	app.queue = queue
 
 	// failure cases
-	delta, err := app.updateAskRepeat("", 0)
+	var delta *resources.Resource
+	delta, err = app.UpdateAskRepeat("", 0)
 	if err == nil || delta != nil {
 		t.Error("empty ask key should not have been found")
 	}
-	delta, err = app.updateAskRepeat("unknown", 0)
+	delta, err = app.UpdateAskRepeat("unknown", 0)
 	if err == nil || delta != nil {
 		t.Error("unknown ask key should not have been found")
 	}
@@ -270,22 +271,22 @@ func TestUpdateRepeat(t *testing.T) {
 	ask := newAllocationAskRepeat(aKey, appID1, res, 1)
 	err = app.AddAllocationAsk(ask)
 	assert.NilError(t, err, "ask should have been added to app")
-	delta, err = app.updateAskRepeat(aKey, 0)
+	delta, err = app.UpdateAskRepeat(aKey, 0)
 	if err != nil || !resources.IsZero(delta) {
 		t.Errorf("0 increase should return zero delta and did not: %v, err %v", delta, err)
 	}
-	delta, err = app.updateAskRepeat(aKey, 1)
+	delta, err = app.UpdateAskRepeat(aKey, 1)
 	if err != nil || !resources.Equals(res, delta) {
 		t.Errorf("increase did not return correct delta, err %v, expected %v got %v", err, res, delta)
 	}
 
 	// decrease to zero
-	delta, err = app.updateAskRepeat(aKey, -2)
+	delta, err = app.UpdateAskRepeat(aKey, -2)
 	if err != nil || !resources.Equals(resources.Multiply(res, -2), delta) {
 		t.Errorf("decrease did not return correct delta, err %v, expected %v got %v", err, resources.Multiply(res, -2), delta)
 	}
 	// decrease to below zero
-	delta, err = app.updateAskRepeat(aKey, -1)
+	delta, err = app.UpdateAskRepeat(aKey, -1)
 	if err == nil || delta != nil {
 		t.Errorf("decrease did not return correct delta, err %v, delta %v", err, delta)
 	}
@@ -821,7 +822,7 @@ func TestQueueUpdate(t *testing.T) {
 	queue, err := createDynamicQueue(root, "test", false)
 	assert.NilError(t, err, "failed to create test queue")
 	app.SetQueue(queue)
-	assert.Equal(t, app.QueueName, "root.test")
+	assert.Equal(t, app.QueuePath, "root.test")
 }
 
 func TestStateTimeOut(t *testing.T) {
@@ -859,6 +860,21 @@ func TestStateTimeOut(t *testing.T) {
 	if !app.stateMachine.Is(Running.String()) || app.stateTimer != nil {
 		t.Fatalf("State is not running or timer was not cleared, state: %s, timer %v", app.stateMachine.Current(), app.stateTimer)
 	}
+
+	startingTimeout = time.Minute * 5
+	app = newApplicationWithTags(appID2, "default", "root.a", map[string]string{AppTagStateAwareDisable: "true"})
+	err = app.HandleApplicationEvent(RunApplication)
+	assert.NilError(t, err, "no error expected new to accepted (timeout test)")
+	err = app.HandleApplicationEvent(RunApplication)
+	assert.NilError(t, err, "no error expected accepted to starting (timeout test)")
+	// give it some time to run and progress
+	time.Sleep(time.Millisecond * 100)
+	if app.IsStarting() {
+		t.Fatal("Starting state should have timed out")
+	}
+	if app.stateTimer != nil {
+		t.Fatalf("Startup timer has not be cleared on time out as expected, %v", app.stateTimer)
+	}
 }
 
 func TestCompleted(t *testing.T) {
@@ -1012,7 +1028,7 @@ func runTimeoutPlaceholderTest(t *testing.T, expectedState string, gangSchedulin
 	// add a second one to check the filter
 	ph = newPlaceholderAlloc(appID1, "uuid-2", nodeID1, "root.a", res)
 	app.AddAllocation(ph)
-	err = common.WaitFor(1*time.Millisecond, 10*time.Millisecond, func() bool {
+	err = common.WaitFor(10*time.Millisecond, 1*time.Second, func() bool {
 		app.RLock()
 		defer app.RUnlock()
 		return app.placeholderTimer == nil
@@ -1060,7 +1076,7 @@ func TestTimeoutPlaceholderAllocReleased(t *testing.T) {
 	alloc := newAllocation(appID1, "real", nodeID1, "root.a", res)
 	app.AddAllocation(alloc)
 	assert.Assert(t, app.IsStarting(), "App should be in starting state after the first allocation")
-	err = common.WaitFor(1*time.Millisecond, 10*time.Millisecond, func() bool {
+	err = common.WaitFor(10*time.Millisecond, 1*time.Second, func() bool {
 		app.RLock()
 		defer app.RUnlock()
 		return app.placeholderTimer == nil
@@ -1109,7 +1125,7 @@ func TestTimeoutPlaceholderCompleting(t *testing.T) {
 	app.RemoveAllocation("uuid-1")
 	assert.Assert(t, app.IsCompleting(), "App should be in completing state all allocs have been removed")
 	// make sure the placeholders time out
-	err = common.WaitFor(1*time.Millisecond, 10*time.Millisecond, func() bool {
+	err = common.WaitFor(10*time.Millisecond, 1*time.Second, func() bool {
 		app.RLock()
 		defer app.RUnlock()
 		return app.placeholderTimer == nil
@@ -1172,3 +1188,15 @@ func TestGetAllRequests(t *testing.T) {
 	assert.Assert(t, len(app.getAllRequests()) == 1, "App should have only one request")
 	assert.Equal(t, app.getAllRequests()[0], ask, "Unexpected request found in the app")
 }
+
+func TestGetQueueNameAfterUnsetQueue(t *testing.T) {
+	app := newApplication(appID1, "default", "root.unknown")
+	assert.Equal(t, app.GetQueuePath(), app.QueuePath)
+	assert.Equal(t, app.GetQueuePath(), "root.unknown")
+
+	// the queue is reset to nil but GetQueuePath should work well
+	app.UnSetQueue()
+	assert.Assert(t, app.queue == nil)
+	assert.Equal(t, app.GetQueuePath(), app.QueuePath)
+	assert.Equal(t, app.GetQueuePath(), "root.unknown")
+}
