@@ -29,10 +29,10 @@ import (
 	"github.com/looplab/fsm"
 	"go.uber.org/zap"
 
+	"github.com/apache/incubator-yunikorn-core/pkg/common"
 	"github.com/apache/incubator-yunikorn-core/pkg/common/configs"
 	"github.com/apache/incubator-yunikorn-core/pkg/common/resources"
 	"github.com/apache/incubator-yunikorn-core/pkg/common/security"
-	"github.com/apache/incubator-yunikorn-core/pkg/interfaces"
 	"github.com/apache/incubator-yunikorn-core/pkg/log"
 	"github.com/apache/incubator-yunikorn-core/pkg/metrics"
 	"github.com/apache/incubator-yunikorn-core/pkg/scheduler/objects"
@@ -51,7 +51,7 @@ type PartitionContext struct {
 	applications           map[string]*objects.Application // applications assigned to this partition
 	completedApplications  map[string]*objects.Application // completed applications from this partition
 	reservedApps           map[string]int                  // applications reserved within this partition, with reservation count
-	nodes                  map[string]*objects.Node        // nodes assigned to this partition
+	nodes                  objects.NodeCollection          // nodes assigned to this partition
 	placementManager       *placement.AppPlacementManager  // placement manager for this partition
 	partitionManager       *partitionManager               // manager for this partition
 	stateMachine           *fsm.FSM                        // the state of the partition for scheduling
@@ -60,7 +60,6 @@ type PartitionContext struct {
 	rules                  *[]configs.PlacementRule        // placement rules to be loaded by the scheduler
 	userGroupCache         *security.UserGroupCache        // user cache per partition
 	totalPartitionResource *resources.Resource             // Total node resources
-	nodeSortingPolicy      *policies.NodeSortingPolicy     // Global Node Sorting Policies
 	allocations            int                             // Number of allocations on the partition
 
 	// The partition write lock must not be held while manipulating an application.
@@ -91,15 +90,13 @@ func newPartitionContext(conf configs.PartitionConfig, rmID string, cc *ClusterC
 		applications:          make(map[string]*objects.Application),
 		completedApplications: make(map[string]*objects.Application),
 		reservedApps:          make(map[string]int),
-		nodes:                 make(map[string]*objects.Node),
-	}
-	pc.partitionManager = &partitionManager{
-		pc: pc,
-		cc: cc,
+		nodes:                 objects.NewNodeCollection(conf.Name),
 	}
+	pc.partitionManager = newPartitionManager(pc, cc)
 	if err := pc.initialPartitionFromConfig(conf); err != nil {
 		return nil, err
 	}
+
 	return pc, nil
 }
 
@@ -134,24 +131,23 @@ func (pc *PartitionContext) initialPartitionFromConfig(conf configs.PartitionCon
 	// get the user group cache for the partition
 	// TODO get the resolver from the config
 	pc.userGroupCache = security.GetUserGroupCache("")
+	pc.updateNodeSortingPolicy(conf)
+	return nil
+}
 
-	// TODO Need some more cleaner interface here.
+// NOTE: this is a lock free call. It should only be called holding the PartitionContext lock.
+func (pc *PartitionContext) updateNodeSortingPolicy(conf configs.PartitionConfig) {
 	var configuredPolicy policies.SortingPolicy
-	configuredPolicy, err = policies.FromString(conf.NodeSortPolicy.Type)
+	configuredPolicy, err := policies.SortingPolicyFromString(conf.NodeSortPolicy.Type)
 	if err != nil {
 		log.Logger().Debug("NodeSorting policy incorrectly set or unknown",
 			zap.Error(err))
-	}
-	switch configuredPolicy {
-	case policies.BinPackingPolicy, policies.FairnessPolicy:
+		log.Logger().Info(fmt.Sprintf("NodeSorting policy not set using '%s' as default", configuredPolicy))
+	} else {
 		log.Logger().Info("NodeSorting policy set from config",
 			zap.String("policyName", configuredPolicy.String()))
-		pc.nodeSortingPolicy = policies.NewNodeSortingPolicy(conf.NodeSortPolicy.Type)
-	case policies.Unknown:
-		log.Logger().Info("NodeSorting policy not set using 'fair' as default")
-		pc.nodeSortingPolicy = policies.NewNodeSortingPolicy("fair")
 	}
-	return nil
+	pc.nodes.SetNodeSortingPolicy(objects.NewNodeSortingPolicy(conf.NodeSortPolicy.Type, conf.NodeSortPolicy.ResourceWeights))
 }
 
 func (pc *PartitionContext) updatePartitionDetails(conf configs.PartitionConfig) error {
@@ -176,11 +172,12 @@ func (pc *PartitionContext) updatePartitionDetails(conf configs.PartitionConfig)
 		// Placing an application will not have a lock on the partition context.
 		pc.placementManager = placement.NewPlacementManager(*pc.rules, pc.GetQueue)
 	}
+	pc.updateNodeSortingPolicy(conf)
 	// start at the root: there is only one queue
 	queueConf := conf.Queues[0]
 	root := pc.root
 	// update the root queue
-	if err := root.SetQueueConfig(queueConf); err != nil {
+	if err := root.ApplyConf(queueConf); err != nil {
 		return err
 	}
 	root.UpdateSortType()
@@ -223,7 +220,7 @@ func (pc *PartitionContext) updateQueues(config []configs.QueueConfig, parent *o
 		if queue == nil {
 			queue, err = objects.NewConfiguredQueue(queueConfig, parent)
 		} else {
-			err = queue.SetQueueConfig(queueConfig)
+			err = queue.ApplyConf(queueConfig)
 		}
 		if err != nil {
 			return err
@@ -306,14 +303,14 @@ func (pc *PartitionContext) AddApplication(app *objects.Application) error {
 	}
 
 	// Put app under the queue
-	queueName := app.QueueName
+	queueName := app.QueuePath
 	pm := pc.getPlacementManager()
 	if pm.IsInitialised() {
 		err := pm.PlaceApplication(app)
 		if err != nil {
 			return fmt.Errorf("failed to place application %s: %v", appID, err)
 		}
-		queueName = app.QueueName
+		queueName = app.QueuePath
 		if queueName == "" {
 			return fmt.Errorf("application rejected by placement rules: %s", appID)
 		}
@@ -487,8 +484,8 @@ func (pc *PartitionContext) GetQueueInfos() dao.QueueDAOInfo {
 // Get the queue info for the whole queue structure to pass to the webservice
 func (pc *PartitionContext) GetPartitionQueues() dao.PartitionQueueDAOInfo {
 	var PartitionQueueDAOInfo = dao.PartitionQueueDAOInfo{}
-	PartitionQueueDAOInfo = pc.root.GetPartitionQueues()
-	PartitionQueueDAOInfo.Partition = pc.Name
+	PartitionQueueDAOInfo = pc.root.GetPartitionQueueDAOInfo()
+	PartitionQueueDAOInfo.Partition = common.GetPartitionNameWithoutClusterID(pc.Name)
 	return PartitionQueueDAOInfo
 }
 
@@ -539,30 +536,7 @@ func (pc *PartitionContext) GetNode(nodeID string) *objects.Node {
 	pc.RLock()
 	defer pc.RUnlock()
 
-	return pc.nodes[nodeID]
-}
-
-// Get a copy of the nodes from the partition.
-// This list does not include reserved nodes or nodes marked unschedulable
-func (pc *PartitionContext) getSchedulableNodes() []*objects.Node {
-	return pc.getNodes(true)
-}
-
-// Get a copy of the nodes from the partition.
-// Excludes unschedulable nodes only, reserved node inclusion depends on the parameter passed in.
-func (pc *PartitionContext) getNodes(excludeReserved bool) []*objects.Node {
-	pc.RLock()
-	defer pc.RUnlock()
-
-	nodes := make([]*objects.Node, 0)
-	for _, node := range pc.nodes {
-		// filter out the nodes that are not scheduling
-		if !node.IsSchedulable() || (excludeReserved && node.IsReserved()) {
-			continue
-		}
-		nodes = append(nodes, node)
-	}
-	return nodes
+	return pc.nodes.GetNode(nodeID)
 }
 
 // Add the node to the partition and process the allocations that are reported by the node.
@@ -585,7 +559,8 @@ func (pc *PartitionContext) AddNode(node *objects.Node, existingAllocations []*o
 	if len(existingAllocations) > 0 {
 		for current, alloc := range existingAllocations {
 			if err := pc.addAllocation(alloc); err != nil {
-				released := pc.removeNode(node.NodeID)
+				// not expecting any inflight replacements on node recovery
+				released, _ := pc.removeNode(node.NodeID)
 				log.Logger().Info("Failed to add existing allocations, changes reversed",
 					zap.String("nodeID", node.NodeID),
 					zap.Int("existingAllocations", len(existingAllocations)),
@@ -622,11 +597,10 @@ func (pc *PartitionContext) updatePartitionResource(delta *resources.Resource) {
 func (pc *PartitionContext) addNodeToList(node *objects.Node) error {
 	pc.Lock()
 	defer pc.Unlock()
-	if pc.nodes[node.NodeID] != nil {
-		return fmt.Errorf("partition %s has an existing node %s, node name must be unique", pc.Name, node.NodeID)
-	}
 	// Node can be added to the system to allow processing of the allocations
-	pc.nodes[node.NodeID] = node
+	if err := pc.nodes.AddNode(node); err != nil {
+		return fmt.Errorf("failed to add node %s to partition %s, error: %v", node.NodeID, pc.Name, err)
+	}
 	metrics.GetSchedulerMetrics().IncActiveNodes()
 
 	// update/set the resources available in the cluster
@@ -649,7 +623,7 @@ func (pc *PartitionContext) addNodeToList(node *objects.Node) error {
 func (pc *PartitionContext) removeNodeFromList(nodeID string) *objects.Node {
 	pc.Lock()
 	defer pc.Unlock()
-	node := pc.nodes[nodeID]
+	node := pc.nodes.RemoveNode(nodeID)
 	if node == nil {
 		log.Logger().Debug("node was not found, node already removed",
 			zap.String("nodeID", nodeID),
@@ -658,7 +632,6 @@ func (pc *PartitionContext) removeNodeFromList(nodeID string) *objects.Node {
 	}
 
 	// Remove node from list of tracked nodes
-	delete(pc.nodes, nodeID)
 	metrics.GetSchedulerMetrics().DecActiveNodes()
 
 	// found the node cleanup the available resources, partition resources cannot be nil at this point
@@ -671,19 +644,22 @@ func (pc *PartitionContext) removeNodeFromList(nodeID string) *objects.Node {
 	return node
 }
 
-// Remove a node from the partition. It returns all removed allocations.
+// Remove a node from the partition. It returns all removed and confirmed allocations.
+// The removed allocations are all linked to the current node.
+// The confirmed allocations are real allocations that are linked to placeholders on the current node and are linked to
+// other nodes.
 // NOTE: this is a lock free call. It must NOT be called holding the PartitionContext lock.
-func (pc *PartitionContext) removeNode(nodeID string) []*objects.Allocation {
+func (pc *PartitionContext) removeNode(nodeID string) ([]*objects.Allocation, []*objects.Allocation) {
 	log.Logger().Info("removing node from partition",
 		zap.String("partition", pc.Name),
 		zap.String("nodeID", nodeID))
 
 	node := pc.removeNodeFromList(nodeID)
 	if node == nil {
-		return nil
+		return nil, nil
 	}
 	// found the node cleanup the allocations linked to the node
-	released := pc.removeNodeAllocations(node)
+	released, confirmed := pc.removeNodeAllocations(node)
 
 	// unreserve all the apps that were reserved on the node
 	reservedKeys, releasedAsks := node.UnReserveApps()
@@ -691,15 +667,16 @@ func (pc *PartitionContext) removeNode(nodeID string) []*objects.Allocation {
 	for i, appID := range reservedKeys {
 		pc.unReserveCount(appID, releasedAsks[i])
 	}
-	return released
+	return released, confirmed
 }
 
 // Remove all allocations that are assigned to a node as part of the node removal. This is not part of the node object
 // as updating the applications and queues is the only goal. Applications and queues are not accessible from the node.
-// The removed allocations are returned.
+// The removed and confirmed allocations are returned.
 // NOTE: this is a lock free call. It must NOT be called holding the PartitionContext lock.
-func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) []*objects.Allocation {
+func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) ([]*objects.Allocation, []*objects.Allocation) {
 	released := make([]*objects.Allocation, 0)
+	confirmed := make([]*objects.Allocation, 0)
 	// walk over all allocations still registered for this node
 	for _, alloc := range node.GetAllAllocations() {
 		allocID := alloc.UUID
@@ -713,6 +690,52 @@ func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) []*objects
 				zap.String("nodeID", node.NodeID))
 			continue
 		}
+		// check for an inflight replacement.
+		if len(alloc.Releases) != 0 {
+			release := alloc.Releases[0]
+			// allocation to update the ask on: this needs to happen on the real alloc never the placeholder
+			askAlloc := alloc
+			// placeholder gets handled differently from normal
+			if alloc.IsPlaceholder() {
+				// Check if the real allocation is made on the same node if not we should trigger a confirmation of
+				// the replacement. Trigger the replacement only if it is NOT on the same node.
+				// If it is on the same node we just keep going as the real allocation will be unlinked as a result of
+				// the removal of this placeholder. The ask update will trigger rescheduling later for the real alloc.
+				if alloc.NodeID != release.NodeID {
+					// ignore the return as that is the same as alloc, the alloc is gone after this call
+					_ = app.ReplaceAllocation(allocID)
+					// track what we confirm on the other node to confirm it in the shim and get is bound
+					confirmed = append(confirmed, release)
+					// the allocation is removed so add it to the list that we return
+					released = append(released, alloc)
+					log.Logger().Info("allocation removed from node and replacement confirmed",
+						zap.String("nodeID", node.NodeID),
+						zap.String("allocationId", allocID),
+						zap.String("replacement nodeID", release.NodeID),
+						zap.String("replacement allocationId", release.UUID))
+					continue
+				}
+				askAlloc = release
+			}
+			// unlink the placeholder and allocation
+			release.Releases = nil
+			alloc.Releases = nil
+			// update the repeat on the real alloc to get it re-scheduled
+			_, err := app.UpdateAskRepeat(askAlloc.Ask.AllocationKey, 1)
+			if err == nil {
+				log.Logger().Info("inflight placeholder replacement reversed due to node removal",
+					zap.String("appID", askAlloc.ApplicationID),
+					zap.String("allocationKey", askAlloc.Ask.AllocationKey),
+					zap.String("nodeID", node.NodeID),
+					zap.String("replacement allocationId", askAlloc.UUID))
+			} else {
+				log.Logger().Error("node removal: repeat update failure for inflight replacement",
+					zap.String("appID", askAlloc.ApplicationID),
+					zap.String("allocationKey", askAlloc.Ask.AllocationKey),
+					zap.String("nodeID", node.NodeID),
+					zap.Error(err))
+			}
+		}
 		// check allocations on the app
 		if app.RemoveAllocation(allocID) == nil {
 			log.Logger().Info("allocation is not found, skipping while removing the node",
@@ -733,9 +756,9 @@ func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) []*objects
 			zap.String("nodeID", node.NodeID),
 			zap.String("allocationId", allocID))
 	}
-	// track the number of allocations
-	pc.updateAllocationCount(-len(released))
-	return released
+	// track the number of allocations: decrement the released allocation AND increment with the confirmed
+	pc.updateAllocationCount(len(confirmed) - len(released))
+	return released, confirmed
 }
 
 func (pc *PartitionContext) calculateOutstandingRequests() []*objects.AllocationAsk {
@@ -883,7 +906,7 @@ func (pc *PartitionContext) reserve(app *objects.Application, node *objects.Node
 
 	log.Logger().Info("allocation ask is reserved",
 		zap.String("appID", appID),
-		zap.String("queue", app.QueueName),
+		zap.String("queue", app.QueuePath),
 		zap.String("allocationKey", ask.AllocationKey),
 		zap.String("node", node.NodeID))
 }
@@ -912,31 +935,16 @@ func (pc *PartitionContext) unReserve(app *objects.Application, node *objects.No
 
 	log.Logger().Info("allocation ask is unreserved",
 		zap.String("appID", appID),
-		zap.String("queue", app.QueueName),
+		zap.String("queue", app.QueuePath),
 		zap.String("allocationKey", ask.AllocationKey),
 		zap.String("node", node.NodeID),
 		zap.Int("reservationsRemoved", num))
 }
 
-// Get the iterator for the sorted nodes list from the partition.
-// Sorting should use a copy of the node list not the main list.
-func (pc *PartitionContext) getNodeIteratorForPolicy(nodes []*objects.Node) interfaces.NodeIterator {
-	configuredPolicy := pc.GetNodeSortingPolicy()
-	if configuredPolicy == policies.Unknown {
-		return nil
-	}
-	// Sort Nodes based on the policy configured.
-	objects.SortNodes(nodes, configuredPolicy)
-	return newDefaultNodeIterator(nodes)
-}
-
-// Create a node iterator for the schedulable nodes based on the policy set for this partition.
-// The iterator is nil if there are no schedulable nodes available.
-func (pc *PartitionContext) GetNodeIterator() interfaces.NodeIterator {
-	if nodeList := pc.getSchedulableNodes(); len(nodeList) != 0 {
-		return pc.getNodeIteratorForPolicy(nodeList)
-	}
-	return nil
+// Create an ordered node iterator based on the node sort policy set for this partition.
+// The iterator is nil if there are no unreserved nodes available.
+func (pc *PartitionContext) GetNodeIterator() objects.NodeIterator {
+	return pc.nodes.GetNodeIterator()
 }
 
 // Update the reservation counter for the app
@@ -982,6 +990,12 @@ func (pc *PartitionContext) GetTotalApplicationCount() int {
 	return len(pc.applications)
 }
 
+func (pc *PartitionContext) GetTotalCompletedApplicationCount() int {
+	pc.RLock()
+	defer pc.RUnlock()
+	return len(pc.completedApplications)
+}
+
 func (pc *PartitionContext) GetTotalAllocationCount() int {
 	pc.RLock()
 	defer pc.RUnlock()
@@ -991,7 +1005,7 @@ func (pc *PartitionContext) GetTotalAllocationCount() int {
 func (pc *PartitionContext) GetTotalNodeCount() int {
 	pc.RLock()
 	defer pc.RUnlock()
-	return len(pc.nodes)
+	return pc.nodes.GetNodeCount()
 }
 
 func (pc *PartitionContext) GetApplications() []*objects.Application {
@@ -1046,11 +1060,7 @@ func (pc *PartitionContext) GetAppsInTerminatedState() []*objects.Application {
 func (pc *PartitionContext) GetNodes() []*objects.Node {
 	pc.RLock()
 	defer pc.RUnlock()
-	var nodeList []*objects.Node
-	for _, node := range pc.nodes {
-		nodeList = append(nodeList, node)
-	}
-	return nodeList
+	return pc.nodes.GetNodes()
 }
 
 // Add an allocation to the partition/node/application/queue during node registration.
@@ -1087,11 +1097,6 @@ func (pc *PartitionContext) addAllocation(alloc *objects.Allocation) error {
 		metrics.GetSchedulerMetrics().IncSchedulingError()
 		return fmt.Errorf("failed to find node %s", alloc.NodeID)
 	}
-	// check the node status again
-	if !node.IsSchedulable() {
-		metrics.GetSchedulerMetrics().IncSchedulingError()
-		return fmt.Errorf("node %s is not in schedulable state", node.NodeID)
-	}
 
 	app := pc.getApplication(alloc.ApplicationID)
 	if app == nil {
@@ -1144,13 +1149,14 @@ func (pc *PartitionContext) convertUGI(ugi *si.UserGroupInformation) (security.U
 //
 // NOTE: this is a lock free call. It must NOT be called holding the PartitionContext lock.
 func (pc *PartitionContext) calculateNodesResourceUsage() map[string][]int {
-	nodesCopy := pc.getNodes(false)
+	nodesCopy := pc.GetNodes()
 	mapResult := make(map[string][]int)
 	for _, node := range nodesCopy {
 		for name, total := range node.GetCapacity().Resources {
 			if float64(total) > 0 {
 				resourceAllocated := float64(node.GetAllocatedResource().Resources[name])
-				v := resourceAllocated / float64(total)
+				// Consider over-allocated node as 100% utilized.
+				v := math.Min(resourceAllocated/float64(total), 1)
 				idx := int(math.Dim(math.Ceil(v*10), 1))
 				if dist, ok := mapResult[name]; !ok {
 					newDist := make([]int, 10)
@@ -1329,10 +1335,24 @@ func (pc *PartitionContext) GetStateTime() time.Time {
 	return pc.stateTime
 }
 
-func (pc *PartitionContext) GetNodeSortingPolicy() policies.SortingPolicy {
+func (pc *PartitionContext) GetNodeSortingPolicyType() policies.SortingPolicy {
 	pc.RLock()
 	defer pc.RUnlock()
-	return pc.nodeSortingPolicy.PolicyType
+	policy := pc.nodes.GetNodeSortingPolicy()
+	if policy == nil {
+		return policies.FairnessPolicy
+	}
+	return policy.PolicyType()
+}
+
+func (pc *PartitionContext) GetNodeSortingResourceWeights() map[string]float64 {
+	pc.RLock()
+	defer pc.RUnlock()
+	policy := pc.nodes.GetNodeSortingPolicy()
+	if policy == nil {
+		return make(map[string]float64)
+	}
+	return policy.ResourceWeights()
 }
 
 func (pc *PartitionContext) moveTerminatedApp(appID string) {
@@ -1354,3 +1374,15 @@ func (pc *PartitionContext) moveTerminatedApp(appID string) {
 	delete(pc.applications, appID)
 	pc.completedApplications[newID] = app
 }
+
+// Check for unlimited nodes in the partition
+func (pc *PartitionContext) hasUnlimitedNode() bool {
+	// We can have only one unlimited node registered
+	if pc.nodes.GetNodeCount() != 1 {
+		return false
+	}
+	for _, v := range pc.nodes.GetNodes() {
+		return v.IsUnlimited()
+	}
+	return false
+}
