@@ -33,6 +33,7 @@ import (
 	"github.com/apache/incubator-yunikorn-core/pkg/common/security"
 	"github.com/apache/incubator-yunikorn-core/pkg/plugins"
 	"github.com/apache/incubator-yunikorn-core/pkg/scheduler/objects"
+	"github.com/apache/incubator-yunikorn-core/pkg/scheduler/policies"
 	"github.com/apache/incubator-yunikorn-scheduler-interface/lib/go/si"
 )
 
@@ -168,7 +169,7 @@ func TestAddNode(t *testing.T) {
 	if err == nil {
 		t.Fatal("nil node add did not return error")
 	}
-	assert.Equal(t, 0, len(partition.nodes), "nil node should not be added")
+	assert.Equal(t, 0, partition.nodes.GetNodeCount(), "nil node should not be added")
 	node := newNodeMaxResource("test1", resources.NewResource())
 	// stop the partition node should be rejected
 	partition.markPartitionForRemoval()
@@ -177,22 +178,22 @@ func TestAddNode(t *testing.T) {
 	if err == nil {
 		t.Error("test node add to draining partition should have failed")
 	}
-	assert.Equal(t, len(partition.nodes), 0, "node list not correct")
+	assert.Equal(t, partition.nodes.GetNodeCount(), 0, "node list not correct")
 
 	// reset the state (hard no checks)
 	partition.stateMachine.SetState(objects.Active.String())
 	err = partition.AddNode(node, nil)
 	assert.NilError(t, err, "test node add failed unexpected")
-	assert.Equal(t, len(partition.nodes), 1, "node list not correct")
+	assert.Equal(t, partition.nodes.GetNodeCount(), 1, "node list not correct")
 	// add the same node nothing changes
 	err = partition.AddNode(node, nil)
 	if err == nil {
 		t.Fatal("add same test node worked unexpected")
 	}
-	assert.Equal(t, len(partition.nodes), 1, "node list not correct")
+	assert.Equal(t, partition.nodes.GetNodeCount(), 1, "node list not correct")
 	err = partition.AddNode(newNodeMaxResource("test2", resources.NewResource()), nil)
 	assert.NilError(t, err, "test node2 add failed unexpected")
-	assert.Equal(t, len(partition.nodes), 2, "node list not correct")
+	assert.Equal(t, partition.nodes.GetNodeCount(), 2, "node list not correct")
 }
 
 func TestAddNodeWithAllocations(t *testing.T) {
@@ -206,12 +207,12 @@ func TestAddNodeWithAllocations(t *testing.T) {
 
 	// add a new app
 	app := newApplication(appID1, "default", defQueue)
-	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1})
+	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"vcore": 1})
 	err = partition.AddApplication(app)
 	assert.NilError(t, err, "add application to partition should not have failed")
 
 	// add a node with allocations
-	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 10})
+	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{"vcore": 10})
 	node := newNodeMaxResource(nodeID1, nodeRes)
 
 	// fail with an unknown app
@@ -222,7 +223,7 @@ func TestAddNodeWithAllocations(t *testing.T) {
 	if err == nil {
 		t.Errorf("add node to partition should have failed (app missing)")
 	}
-	assert.Equal(t, len(partition.nodes), 0, "error returned but node still added to the partition (app)")
+	assert.Equal(t, partition.nodes.GetNodeCount(), 0, "error returned but node still added to the partition (app)")
 
 	// fail with a broken alloc
 	ask = newAllocationAsk("alloc-1", appID1, appRes)
@@ -232,7 +233,7 @@ func TestAddNodeWithAllocations(t *testing.T) {
 	if err == nil {
 		t.Errorf("add node to partition should have failed (uuid missing)")
 	}
-	assert.Equal(t, len(partition.nodes), 0, "error returned but node still added to the partition (uuid)")
+	assert.Equal(t, partition.nodes.GetNodeCount(), 0, "error returned but node still added to the partition (uuid)")
 
 	// fix the alloc add the node will work now
 	alloc = objects.NewAllocation("alloc-1-uuid", nodeID1, ask)
@@ -241,7 +242,7 @@ func TestAddNodeWithAllocations(t *testing.T) {
 	err = partition.AddNode(node, allocs)
 	// check the partition
 	assert.NilError(t, err, "add node to partition should not have failed")
-	assert.Equal(t, len(partition.nodes), 1, "no error returned but node not added to the partition")
+	assert.Equal(t, partition.nodes.GetNodeCount(), 1, "no error returned but node not added to the partition")
 	assert.Assert(t, resources.Equals(nodeRes, partition.GetTotalPartitionResource()), "add node to partition did not update total resources expected %v got %d", nodeRes, partition.GetTotalPartitionResource())
 	assert.Equal(t, partition.GetTotalAllocationCount(), 1, "add node to partition did not add allocation")
 
@@ -254,16 +255,16 @@ func TestRemoveNode(t *testing.T) {
 	assert.NilError(t, err, "test partition create failed with error")
 	err = partition.AddNode(newNodeMaxResource("test", resources.NewResource()), nil)
 	assert.NilError(t, err, "test node add failed unexpected")
-	assert.Equal(t, 1, len(partition.nodes), "node list not correct")
+	assert.Equal(t, 1, partition.nodes.GetNodeCount(), "node list not correct")
 
 	// remove non existing node
-	_ = partition.removeNode("")
-	assert.Equal(t, 1, len(partition.nodes), "nil node should not remove anything")
-	_ = partition.removeNode("does not exist")
-	assert.Equal(t, 1, len(partition.nodes), "non existing node was removed")
+	_, _ = partition.removeNode("")
+	assert.Equal(t, 1, partition.nodes.GetNodeCount(), "nil node should not remove anything")
+	_, _ = partition.removeNode("does not exist")
+	assert.Equal(t, 1, partition.nodes.GetNodeCount(), "non existing node was removed")
 
-	_ = partition.removeNode("test")
-	assert.Equal(t, 0, len(partition.nodes), "node was not removed")
+	_, _ = partition.removeNode("test")
+	assert.Equal(t, 0, partition.nodes.GetNodeCount(), "node was not removed")
 }
 
 func TestRemoveNodeWithAllocations(t *testing.T) {
@@ -276,9 +277,9 @@ func TestRemoveNodeWithAllocations(t *testing.T) {
 	assert.NilError(t, err, "add application to partition should not have failed")
 
 	// add a node with allocations: must have the correct app added already
-	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1000})
+	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{"vcore": 1000})
 	node := newNodeMaxResource(nodeID1, nodeRes)
-	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1})
+	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"vcore": 1})
 	ask := newAllocationAsk("alloc-1", appID1, appRes)
 	allocUUID := "alloc-1-uuid"
 	alloc := objects.NewAllocation(allocUUID, nodeID1, ask)
@@ -287,7 +288,7 @@ func TestRemoveNodeWithAllocations(t *testing.T) {
 	assert.NilError(t, err, "add node to partition should not have failed")
 	// get what was allocated
 	allocated := node.GetAllAllocations()
-	assert.Equal(t, 1, len(allocated), "allocation not added correctly expected 1 got: %v", allocated)
+	assert.Equal(t, 1, len(allocated), "allocation not added correctly")
 
 	// add broken allocations
 	ask = newAllocationAsk("alloc-na", "not-an-app", appRes)
@@ -298,69 +299,227 @@ func TestRemoveNodeWithAllocations(t *testing.T) {
 	node.AddAllocation(alloc)
 
 	// remove the node this cannot fail
-	released := partition.removeNode(nodeID1)
-	assert.Equal(t, 0, partition.GetTotalNodeCount(), "node list was not updated, node was not removed expected 0 got %d", partition.GetTotalNodeCount())
-	assert.Equal(t, 1, len(released), "node did not release correct allocation expected 1 got %d", len(released))
+	released, confirmed := partition.removeNode(nodeID1)
+	assert.Equal(t, 0, partition.GetTotalNodeCount(), "node list was not updated, node was not removed")
+	assert.Equal(t, 1, len(released), "node did not release correct allocation")
+	assert.Equal(t, 0, len(confirmed), "node did not confirm correct allocation")
 	assert.Equal(t, released[0].UUID, allocUUID, "UUID returned by release not the same as on allocation")
 }
 
-func TestGetNodes(t *testing.T) {
+// test with a replacement of a placeholder: placeholder and real on the same node that gets removed
+func TestRemoveNodeWithPlaceholders(t *testing.T) {
 	partition, err := newBasePartition()
-	assert.NilError(t, err, "test partition create failed with error")
+	assert.NilError(t, err, "partition create failed")
 
-	nodes := partition.getSchedulableNodes()
-	assert.Equal(t, 0, len(nodes), "list should have been empty")
-
-	err = partition.AddNode(newNodeMaxResource(nodeID1, resources.NewResource()), nil)
-	assert.NilError(t, err, "test node1 add failed unexpected")
-
-	err = partition.AddNode(newNodeMaxResource(nodeID2, resources.NewResource()), nil)
-	assert.NilError(t, err, "test node2 add failed unexpected")
-	// add one node that will not be returned in the node list
-	node3 := "notScheduling"
-	node := newNodeMaxResource(node3, resources.NewResource())
-	node.SetSchedulable(false)
-	err = partition.AddNode(node, nil)
-	assert.NilError(t, err, "test node3 add failed unexpected")
-	node4 := "reserved"
-	node4Max := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 10})
-	err = partition.AddNode(newNodeMaxResource(node4, node4Max), nil)
-	assert.NilError(t, err, "test node4 add failed unexpected")
-	// get individual nodes
-	node = partition.GetNode("")
-	var nilNode *objects.Node
-	assert.Equal(t, nilNode, node, "retrieved non existing node with no name")
-	node = partition.GetNode("does not exist")
-	assert.Equal(t, nilNode, node, "existing node returned for non existing name")
-	node = partition.GetNode(node3)
-	assert.Equal(t, node3, node.NodeID, "failed to retrieve correct node3")
-	node = partition.GetNode(node4)
-	assert.Equal(t, node4, node.NodeID, "failed to retrieve correct node4")
-	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1})
-	ask := newAllocationAsk("alloc-1", appID1, appRes)
+	// add a new app
 	app := newApplication(appID1, "default", defQueue)
-	err = node.Reserve(app, ask)
-	assert.NilError(t, err, "reserve on node4 should not have failed")
+	err = partition.AddApplication(app)
+	assert.NilError(t, err, "add application to partition should not have failed")
 
-	assert.Equal(t, 4, len(partition.nodes), "node list not correct length")
-	nodes = partition.getSchedulableNodes()
-	// returned list should be only two long
-	assert.Equal(t, 2, len(nodes), "node list not filtered")
-	// map iteration is random so don't know which we get first
-	for _, node = range nodes {
-		if node.NodeID != nodeID1 && node.NodeID != nodeID2 {
-			t.Fatalf("unexpected node returned in list: %s", node.NodeID)
-		}
-	}
-	nodes = partition.getNodes(false)
-	// returned list should be 3 long: no reserved filter
-	assert.Equal(t, 3, len(nodes), "node list was incorrectly filtered")
-	// check if we have all nodes: since there is a backing map we cannot have duplicates
-	for _, node = range nodes {
-		if node.NodeID == node3 {
-			t.Fatalf("unexpected node returned in list: %s", node.NodeID)
-		}
-	}
+	// add a node with allocation: must have the correct app added already
+	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 10})
+	node1 := newNodeMaxResource(nodeID1, nodeRes)
+	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1})
+	ask := newAllocationAskTG("placeholder", appID1, taskGroup, appRes, true)
+	ph := objects.NewAllocation(phID, nodeID1, ask)
+	allocs := []*objects.Allocation{ph}
+	err = partition.AddNode(node1, allocs)
+	assert.NilError(t, err, "add node1 to partition should not have failed")
+	// get what was allocated
+	allocated := node1.GetAllAllocations()
+	assert.Equal(t, 1, len(allocated), "allocation not added correctly to node1 expected 1 got: %v", allocated)
+	assert.Assert(t, resources.Equals(node1.GetAllocatedResource(), appRes), "allocation not added correctly to node1")
+
+	// fake an ask that is used
+	ask = newAllocationAskAll("real", appID1, taskGroup, appRes, 1, 1, false)
+	err = app.AddAllocationAsk(ask)
+	assert.NilError(t, err, "ask should be added to app")
+	_, err = app.UpdateAskRepeat("real", -1)
+	assert.NilError(t, err, "ask should have been updated without error")
+	assert.Assert(t, resources.IsZero(app.GetPendingResource()), "app should not have pending resources")
+
+	// add real allocation that is replacing the placeholder
+	alloc := objects.NewAllocation(allocID, nodeID1, ask)
+	alloc.Releases = []*objects.Allocation{ph}
+	// double link as if the replacement is ongoing
+	ph.Releases = []*objects.Allocation{alloc}
+	alloc.Result = objects.Replaced
+
+	allocs = app.GetAllAllocations()
+	assert.Equal(t, len(allocs), 1, "expected one allocation for the app (placeholder)")
+
+	// remove the node that has both placeholder and real allocation
+	released, confirmed := partition.removeNode(nodeID1)
+	assert.Equal(t, 0, partition.GetTotalNodeCount(), "node list was not updated, node was not removed")
+	assert.Equal(t, 1, len(released), "node removal did not release correct allocation")
+	assert.Equal(t, 0, len(confirmed), "node removal should not have confirmed allocation")
+	assert.Equal(t, phID, released[0].UUID, "UUID returned by release not the same as the placeholder")
+	allocs = app.GetAllAllocations()
+	assert.Equal(t, 0, len(allocs), "expected no allocations for the app")
+	assert.Assert(t, resources.Equals(app.GetPendingResource(), appRes), "app should have updated pending resources")
+	// check the interim state of the placeholder involved
+	assert.Equal(t, 0, len(ph.Releases), "placeholder should have no releases linked anymore")
+}
+
+func TestCalculateNodesResourceUsage(t *testing.T) {
+	partition, err := newBasePartition()
+	assert.NilError(t, err, "partition create failed")
+	oldCapacity := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 100})
+	node := newNodeMaxResource(nodeID1, oldCapacity)
+	err = partition.AddNode(node, nil)
+	assert.NilError(t, err)
+
+	occupiedResources := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 50})
+	alloc := objects.NewAllocation(allocID, nodeID1, newAllocationAsk("key", "appID", occupiedResources))
+	node.AddAllocation(alloc)
+	usageMap := partition.calculateNodesResourceUsage()
+	assert.Equal(t, node.GetAvailableResource().Resources["first"], resources.Quantity(50))
+	assert.Equal(t, usageMap["first"][4], 1)
+
+	occupiedResources = resources.NewResourceFromMap(map[string]resources.Quantity{"first": 50})
+	alloc = objects.NewAllocation(allocID, nodeID1, newAllocationAsk("key", "appID", occupiedResources))
+	node.AddAllocation(alloc)
+	usageMap = partition.calculateNodesResourceUsage()
+	assert.Equal(t, node.GetAvailableResource().Resources["first"], resources.Quantity(0))
+	assert.Equal(t, usageMap["first"][9], 1)
+
+	newCapacity := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 80})
+	node.SetCapacity(newCapacity)
+	usageMap = partition.calculateNodesResourceUsage()
+	assert.Assert(t, node.GetAvailableResource().HasNegativeValue())
+	assert.Equal(t, usageMap["first"][9], 1)
+}
+
+// test with a replacement of a placeholder: placeholder on the removed node, real on the 2nd node
+func TestRemoveNodeWithReplacement(t *testing.T) {
+	partition, err := newBasePartition()
+	assert.NilError(t, err, "partition create failed")
+
+	// add a new app
+	app := newApplication(appID1, "default", defQueue)
+	err = partition.AddApplication(app)
+	assert.NilError(t, err, "add application to partition should not have failed")
+
+	// add a node with allocation: must have the correct app added already
+	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 10})
+	node1 := newNodeMaxResource(nodeID1, nodeRes)
+	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1})
+	ask := newAllocationAskAll("placeholder", appID1, taskGroup, appRes, 0, 1, true)
+	ph := objects.NewAllocation(phID, nodeID1, ask)
+	allocs := []*objects.Allocation{ph}
+	err = partition.AddNode(node1, allocs)
+	assert.NilError(t, err, "add node1 to partition should not have failed")
+	// get what was allocated
+	allocated := node1.GetAllAllocations()
+	assert.Equal(t, 1, len(allocated), "allocation not added correctly to node1")
+	assert.Assert(t, resources.Equals(node1.GetAllocatedResource(), appRes), "allocation not added correctly to node1")
+
+	node2 := newNodeMaxResource(nodeID2, nodeRes)
+	err = partition.AddNode(node2, nil)
+	assert.NilError(t, err, "add node2 to partition should not have failed")
+	assert.Equal(t, 2, partition.GetTotalNodeCount(), "node list was not updated as expected")
+
+	// fake an ask that is used
+	ask = newAllocationAskAll("real", appID1, taskGroup, appRes, 1, 1, false)
+	err = app.AddAllocationAsk(ask)
+	assert.NilError(t, err, "ask should be added to app")
+	_, err = app.UpdateAskRepeat("real", -1)
+	assert.NilError(t, err, "ask should have been updated without error")
+	assert.Assert(t, resources.IsZero(app.GetPendingResource()), "app should not have pending resources")
+
+	// add real allocation that is replacing the placeholder on the 2nd node
+	alloc := objects.NewAllocation(allocID, nodeID2, ask)
+	alloc.Releases = []*objects.Allocation{ph}
+	alloc.Result = objects.Replaced
+	node2.AddAllocation(alloc)
+	allocated = node2.GetAllAllocations()
+	assert.Equal(t, 1, len(allocated), "allocation not added correctly to node2")
+	assert.Assert(t, resources.Equals(node2.GetAllocatedResource(), appRes), "allocation not added correctly to node2 (resource count)")
+
+	// double link as if the replacement is ongoing
+	ph.Releases = []*objects.Allocation{alloc}
+
+	allocs = app.GetAllAllocations()
+	assert.Equal(t, len(allocs), 1, "expected one allocation for the app (placeholder)")
+
+	// remove the node with the placeholder
+	released, confirmed := partition.removeNode(nodeID1)
+	assert.Equal(t, 1, partition.GetTotalNodeCount(), "node list was not updated, node was not removed")
+	assert.Equal(t, 1, len(node2.GetAllAllocations()), "remaining node should have allocation")
+	assert.Equal(t, 1, len(released), "node removal did not release correct allocation")
+	assert.Equal(t, 1, len(confirmed), "node removal did not confirm correct allocation")
+	assert.Equal(t, phID, released[0].UUID, "UUID returned by release not the same as the placeholder")
+	assert.Equal(t, allocID, confirmed[0].UUID, "UUID returned by confirmed not the same as the real allocation")
+	assert.Assert(t, resources.IsZero(app.GetPendingResource()), "app should not have pending resources")
+	allocs = app.GetAllAllocations()
+	assert.Equal(t, 1, len(allocs), "expected one allocation for the app (real)")
+	assert.Equal(t, allocID, allocs[0].UUID, "UUID for the app is not the same as the real allocation")
+	assert.Equal(t, objects.Allocated, allocs[0].Result, "allocation state should be allocated")
+	assert.Equal(t, 0, len(allocs[0].Releases), "real allocation should have no releases linked anymore")
+}
+
+// test with a replacement of a placeholder: real on the removed node placeholder on the 2nd node
+func TestRemoveNodeWithReal(t *testing.T) {
+	partition, err := newBasePartition()
+	assert.NilError(t, err, "partition create failed")
+
+	// add a new app
+	app := newApplication(appID1, "default", defQueue)
+	err = partition.AddApplication(app)
+	assert.NilError(t, err, "add application to partition should not have failed")
+
+	// add a node with allocation: must have the correct app added already
+	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 10})
+	node1 := newNodeMaxResource(nodeID1, nodeRes)
+	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1})
+	ask := newAllocationAskAll("placeholder", appID1, taskGroup, appRes, 0, 1, true)
+	ph := objects.NewAllocation(phID, nodeID1, ask)
+	allocs := []*objects.Allocation{ph}
+	err = partition.AddNode(node1, allocs)
+	assert.NilError(t, err, "add node1 to partition should not have failed")
+	// get what was allocated
+	allocated := node1.GetAllAllocations()
+	assert.Equal(t, 1, len(allocated), "allocation not added correctly to node1")
+	assert.Assert(t, resources.Equals(node1.GetAllocatedResource(), appRes), "allocation not added correctly to node1")
+
+	node2 := newNodeMaxResource(nodeID2, nodeRes)
+	err = partition.AddNode(node2, nil)
+	assert.NilError(t, err, "add node2 to partition should not have failed")
+	assert.Equal(t, 2, partition.GetTotalNodeCount(), "node list was not updated as expected")
+
+	// fake an ask that is used
+	ask = newAllocationAskAll("real", appID1, taskGroup, appRes, 1, 1, false)
+	err = app.AddAllocationAsk(ask)
+	assert.NilError(t, err, "ask should be added to app")
+	_, err = app.UpdateAskRepeat("real", -1)
+	assert.NilError(t, err, "ask should have been updated without error")
+	assert.Assert(t, resources.IsZero(app.GetPendingResource()), "app should not have pending resources")
+
+	// add real allocation that is replacing the placeholder on the 2nd node
+	alloc := objects.NewAllocation(allocID, nodeID2, ask)
+	alloc.Releases = []*objects.Allocation{ph}
+	alloc.Result = objects.Replaced
+	node2.AddAllocation(alloc)
+	allocated = node2.GetAllAllocations()
+	assert.Equal(t, 1, len(allocated), "allocation not added correctly to node2")
+	assert.Assert(t, resources.Equals(node2.GetAllocatedResource(), appRes), "allocation not added correctly to node2 (resource count)")
+
+	// double link as if the replacement is ongoing
+	ph.Releases = []*objects.Allocation{alloc}
+
+	allocs = app.GetAllAllocations()
+	assert.Equal(t, len(allocs), 1, "expected one allocation for the app (placeholder)")
+
+	// remove the node with the real allocation
+	released, confirmed := partition.removeNode(nodeID2)
+	assert.Equal(t, 1, partition.GetTotalNodeCount(), "node list was not updated, node was not removed")
+	assert.Equal(t, 0, len(released), "node removal did not release correct allocation")
+	assert.Equal(t, 0, len(confirmed), "node removal did not confirm correct allocation")
+	assert.Assert(t, resources.Equals(app.GetPendingResource(), appRes), "app should have updated pending resources")
+	allocs = app.GetAllAllocations()
+	assert.Equal(t, 1, len(allocs), "expected one allocation for the app (placeholder")
+	assert.Equal(t, phID, allocs[0].UUID, "UUID for the app is not the same as the real allocation")
+	assert.Equal(t, 0, len(ph.Releases), "no inflight replacements linked")
 }
 
 func TestAddApp(t *testing.T) {
@@ -403,7 +562,7 @@ func TestAddAppTaskGroup(t *testing.T) {
 	assert.NilError(t, err, "partition create failed")
 
 	// add a new app: TG specified with resource no max set on the queue
-	task := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 10})
+	task := resources.NewResourceFromMap(map[string]resources.Quantity{"vcore": 10})
 	app := newApplicationTG(appID1, "default", defQueue, task)
 	assert.Assert(t, resources.Equals(app.GetPlaceholderAsk(), task), "placeholder ask not set as expected")
 	// queue sort policy is FIFO this should work
@@ -415,7 +574,7 @@ func TestAddAppTaskGroup(t *testing.T) {
 
 	// queue now has fair as sort policy app add should fail
 	queue := partition.GetQueue(defQueue)
-	err = queue.SetQueueConfig(configs.QueueConfig{
+	err = queue.ApplyConf(configs.QueueConfig{
 		Name:       "default",
 		Parent:     false,
 		Queues:     nil,
@@ -429,12 +588,12 @@ func TestAddAppTaskGroup(t *testing.T) {
 	}
 
 	// queue with stateaware as sort policy, with a max set smaller than placeholder ask: app add should fail
-	err = queue.SetQueueConfig(configs.QueueConfig{
+	err = queue.ApplyConf(configs.QueueConfig{
 		Name:       "default",
 		Parent:     false,
 		Queues:     nil,
 		Properties: map[string]string{configs.ApplicationSortPolicy: "stateaware"},
-		Resources:  configs.Resources{Max: map[string]string{"first": "5"}},
+		Resources:  configs.Resources{Max: map[string]string{"vcore": "5"}},
 	})
 	assert.NilError(t, err, "updating queue should not have failed (stateaware & max)")
 	queue.UpdateSortType()
@@ -444,11 +603,11 @@ func TestAddAppTaskGroup(t *testing.T) {
 	}
 
 	// queue with stateaware as sort policy, with a max set larger than placeholder ask: app add works
-	err = queue.SetQueueConfig(configs.QueueConfig{
+	err = queue.ApplyConf(configs.QueueConfig{
 		Name:      "default",
 		Parent:    false,
 		Queues:    nil,
-		Resources: configs.Resources{Max: map[string]string{"first": "20"}},
+		Resources: configs.Resources{Max: map[string]string{"vcore": "20"}},
 	})
 	assert.NilError(t, err, "updating queue should not have failed (max resource)")
 	err = partition.AddApplication(app)
@@ -465,7 +624,7 @@ func TestRemoveApp(t *testing.T) {
 	err = partition.AddApplication(app)
 	assert.NilError(t, err, "add application to partition should not have failed")
 	// add a node to allow adding an allocation
-	node1 := newNodeMaxResource(nodeID1, resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1000}))
+	node1 := newNodeMaxResource(nodeID1, resources.NewResourceFromMap(map[string]resources.Quantity{"vcore": 1000}))
 	// add a node this must work
 	err = partition.AddNode(node1, nil)
 	assert.NilError(t, err, "add node to partition should not have failed")
@@ -473,7 +632,7 @@ func TestRemoveApp(t *testing.T) {
 		t.Fatalf("node not added to partition as expected (node nil)")
 	}
 
-	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1})
+	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"vcore": 1})
 	ask := newAllocationAsk("alloc-nr", appNotRemoved, appRes)
 	uuid := "alloc-nr-uuid"
 	alloc := objects.NewAllocation(uuid, nodeID1, ask)
@@ -523,14 +682,14 @@ func TestRemoveAppAllocs(t *testing.T) {
 	err = partition.AddApplication(app)
 	assert.NilError(t, err, "add application to partition should not have failed")
 	// add a node to allow adding an allocation
-	node1 := newNodeMaxResource(nodeID1, resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1000}))
+	node1 := newNodeMaxResource(nodeID1, resources.NewResourceFromMap(map[string]resources.Quantity{"vcore": 1000}))
 	// add a node this must work
 	err = partition.AddNode(node1, nil)
 	assert.NilError(t, err, "add node to partition should not have failed")
 	if partition.GetNode(nodeID1) == nil {
 		t.Fatalf("node not added to partition as expected (node nil)")
 	}
-	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1})
+	appRes := resources.NewResourceFromMap(map[string]resources.Quantity{"vcore": 1})
 	ask := newAllocationAsk("alloc-nr", appNotRemoved, appRes)
 	alloc := objects.NewAllocation("alloc-nr-uuid", nodeID1, ask)
 	err = partition.addAllocation(alloc)
@@ -701,7 +860,7 @@ func TestUpdateQueues(t *testing.T) {
 	assert.Assert(t, def.IsDraining(), "'root.default' queue should have been marked for removal")
 
 	var resExpect *resources.Resource
-	resMap := map[string]string{"first": "1"}
+	resMap := map[string]string{"vcore": "1"}
 	resExpect, err = resources.NewResourceFromConf(resMap)
 	assert.NilError(t, err, "resource from conf failed")
 
@@ -775,7 +934,7 @@ func TestTryAllocate(t *testing.T) {
 	// the ask with the higher priority is the second one added alloc-2 for app-1
 	app := newApplication(appID1, "default", "root.parent.sub-leaf")
 
-	res, err := resources.NewResourceFromConf(map[string]string{"first": "1"})
+	res, err := resources.NewResourceFromConf(map[string]string{"vcore": "1"})
 	assert.NilError(t, err, "failed to create resource")
 
 	// add to the partition
@@ -838,7 +997,7 @@ func TestTryAllocateLarge(t *testing.T) {
 	objects.SetReservationDelay(10 * time.Nanosecond)
 	defer objects.SetReservationDelay(2 * time.Second)
 
-	res, err := resources.NewResourceFromConf(map[string]string{"first": "100"})
+	res, err := resources.NewResourceFromConf(map[string]string{"vcore": "100"})
 	assert.NilError(t, err, "failed to create resource")
 
 	app := newApplication(appID1, "default", "root.parent.sub-leaf")
@@ -873,7 +1032,7 @@ func TestAllocReserveNewNode(t *testing.T) {
 	node2 := partition.GetNode(nodeID2)
 	node2.SetSchedulable(false)
 
-	res, err := resources.NewResourceFromConf(map[string]string{"first": "8"})
+	res, err := resources.NewResourceFromConf(map[string]string{"vcore": "8"})
 	assert.NilError(t, err, "failed to create resource")
 
 	// only one resource for alloc fits on a node
@@ -928,7 +1087,7 @@ func TestTryAllocateReserve(t *testing.T) {
 	// sub-leaf will have an app with 2 requests and thus more unconfirmed resources compared to leaf2
 	// this should filter up in the parent and the 1st allocate should show an app-1 allocation
 	// the ask with the higher priority is the second one added alloc-2 for app-1
-	res, err := resources.NewResourceFromConf(map[string]string{"first": "1"})
+	res, err := resources.NewResourceFromConf(map[string]string{"vcore": "1"})
 	assert.NilError(t, err, "failed to create resource")
 
 	app := newApplication(appID1, "default", "root.parent.sub-leaf")
@@ -993,7 +1152,7 @@ func TestTryAllocateWithReserved(t *testing.T) {
 		t.Fatalf("empty cluster reserved allocate returned allocation: %v", alloc)
 	}
 
-	res, err := resources.NewResourceFromConf(map[string]string{"first": "5"})
+	res, err := resources.NewResourceFromConf(map[string]string{"vcore": "5"})
 	assert.NilError(t, err, "failed to create resource")
 
 	app := newApplication(appID1, "default", "root.parent.sub-leaf")
@@ -1046,7 +1205,7 @@ func TestScheduleRemoveReservedAsk(t *testing.T) {
 	objects.SetReservationDelay(10 * time.Nanosecond)
 	defer objects.SetReservationDelay(2 * time.Second)
 
-	res, err := resources.NewResourceFromConf(map[string]string{"first": "4"})
+	res, err := resources.NewResourceFromConf(map[string]string{"vcore": "4"})
 	assert.NilError(t, err, "resource creation failed")
 	app := newApplication(appID1, "default", "root.parent.sub-leaf")
 	err = partition.AddApplication(app)
@@ -1123,7 +1282,7 @@ func TestUpdateRootQueue(t *testing.T) {
 	if partition == nil {
 		t.Fatal("partition create failed")
 	}
-	res, err := resources.NewResourceFromConf(map[string]string{"first": "20"})
+	res, err := resources.NewResourceFromConf(map[string]string{"vcore": "20"})
 	assert.NilError(t, err, "resource creation failed")
 	assert.Assert(t, resources.Equals(res, partition.totalPartitionResource), "partition resource not set as expected")
 	assert.Assert(t, resources.Equals(res, partition.root.GetMaxResource()), "root max resource not set as expected")
@@ -1208,7 +1367,7 @@ func TestUpdateNode(t *testing.T) {
 
 	err = partition.AddNode(newNodeMaxResource("test", newRes), nil)
 	assert.NilError(t, err, "test node add failed unexpected")
-	assert.Equal(t, 1, len(partition.nodes), "node list not correct")
+	assert.Equal(t, 1, partition.nodes.GetNodeCount(), "node list not correct")
 
 	if !resources.Equals(newRes, partition.GetTotalPartitionResource()) {
 		t.Errorf("Expected partition resource %s, doesn't match with actual partition resource %s", newRes, partition.GetTotalPartitionResource())
@@ -1250,12 +1409,12 @@ func TestUpdateNode(t *testing.T) {
 }
 
 func TestAddTGApplication(t *testing.T) {
-	limit := map[string]string{"first": "1"}
+	limit := map[string]string{"vcore": "1"}
 	partition, err := newLimitedPartition(limit)
 	assert.NilError(t, err, "partition create failed")
 	// add a app with TG that does not fit in the queue
 	var tgRes *resources.Resource
-	tgRes, err = resources.NewResourceFromConf(map[string]string{"first": "10"})
+	tgRes, err = resources.NewResourceFromConf(map[string]string{"vcore": "10"})
 	assert.NilError(t, err, "failed to create resource")
 	app := newApplicationTG(appID1, "default", "root.limited", tgRes)
 	err = partition.AddApplication(app)
@@ -1264,7 +1423,7 @@ func TestAddTGApplication(t *testing.T) {
 	}
 
 	// add a app with TG that does fit in the queue
-	limit = map[string]string{"first": "100"}
+	limit = map[string]string{"vcore": "100"}
 	partition, err = newLimitedPartition(limit)
 	assert.NilError(t, err, "partition create failed")
 	err = partition.AddApplication(app)
@@ -1285,23 +1444,23 @@ func TestAddTGAppDynamic(t *testing.T) {
 	assert.NilError(t, err, "partition create failed")
 	// add a app with TG that does fit in the dynamic queue (no limit)
 	var tgRes *resources.Resource
-	tgRes, err = resources.NewResourceFromConf(map[string]string{"first": "10"})
+	tgRes, err = resources.NewResourceFromConf(map[string]string{"vcore": "10"})
 	assert.NilError(t, err, "failed to create resource")
 	tags := map[string]string{"taskqueue": "unlimited"}
 	app := newApplicationTGTags(appID1, "default", "unknown", tgRes, tags)
 	err = partition.AddApplication(app)
 	assert.NilError(t, err, "app-1 should have been added to the partition")
-	assert.Equal(t, app.GetQueueName(), "root.unlimited", "app-1 not placed in expected queue")
+	assert.Equal(t, app.GetQueuePath(), "root.unlimited", "app-1 not placed in expected queue")
 
-	jsonRes := "{\"resources\":{\"first\":{\"value\":10}}}"
+	jsonRes := "{\"resources\":{\"vcore\":{\"value\":10}}}"
 	tags = map[string]string{"taskqueue": "same", objects.AppTagNamespaceResourceQuota: jsonRes}
 	app = newApplicationTGTags(appID2, "default", "unknown", tgRes, tags)
 	err = partition.AddApplication(app)
 	assert.NilError(t, err, "app-2 should have been added to the partition")
 	assert.Equal(t, partition.getApplication(appID2), app, "partition failed to add app incorrect app returned")
-	assert.Equal(t, app.GetQueueName(), "root.same", "app-2 not placed in expected queue")
+	assert.Equal(t, app.GetQueuePath(), "root.same", "app-2 not placed in expected queue")
 
-	jsonRes = "{\"resources\":{\"first\":{\"value\":1}}}"
+	jsonRes = "{\"resources\":{\"vcore\":{\"value\":1}}}"
 	tags = map[string]string{"taskqueue": "smaller", objects.AppTagNamespaceResourceQuota: jsonRes}
 	app = newApplicationTGTags(appID3, "default", "unknown", tgRes, tags)
 	err = partition.AddApplication(app)
@@ -1330,11 +1489,11 @@ func TestPlaceholderAndRealAllocationResMismatch(t *testing.T) {
 
 	var tgRes, res *resources.Resource
 	var res1 *resources.Resource
-	tgRes, err = resources.NewResourceFromConf(map[string]string{"first": "10"})
+	tgRes, err = resources.NewResourceFromConf(map[string]string{"vcore": "10"})
 	assert.NilError(t, err, "failed to create resource")
-	res, err = resources.NewResourceFromConf(map[string]string{"first": "1"})
+	res, err = resources.NewResourceFromConf(map[string]string{"vcore": "1"})
 	assert.NilError(t, err, "failed to create resource")
-	res1, err = resources.NewResourceFromConf(map[string]string{"first": "2"})
+	res1, err = resources.NewResourceFromConf(map[string]string{"vcore": "2"})
 	assert.NilError(t, err, "failed to create resource")
 
 	// add node to allow allocation
@@ -1350,7 +1509,6 @@ func TestPlaceholderAndRealAllocationResMismatch(t *testing.T) {
 	err = partition.AddApplication(app)
 	assert.NilError(t, err, "app-1 should have been added to the partition")
 	// add an ask for a placeholder and allocate
-	const taskGroup = "tg-1"
 	ask := newAllocationAskTG(phID, appID1, taskGroup, res, true)
 	err = app.AddAllocationAsk(ask)
 	assert.NilError(t, err, "failed to add placeholder ask ph-1 to app")
@@ -1389,7 +1547,7 @@ func TestPlaceholderAndRealAllocationResMismatch(t *testing.T) {
 	assert.Equal(t, record.Type, si.EventRecord_REQUEST)
 	assert.Equal(t, record.ObjectID, "real-2")
 	assert.Equal(t, record.GroupID, "app-1")
-	assert.Equal(t, record.Message, "Real Pod real-2 allocation [first:2] is not matching with placeholder ph-1 allocation [first:1] in application app-1")
+	assert.Equal(t, record.Message, "Real Pod real-2 allocation [vcore:2] is not matching with placeholder ph-1 allocation [vcore:1] in application app-1")
 	assert.Equal(t, record.Reason, "Resource Allocation Mismatch between real and placeholder")
 }
 
@@ -1402,9 +1560,9 @@ func TestTryPlaceholderAllocate(t *testing.T) {
 	}
 
 	var tgRes, res *resources.Resource
-	tgRes, err = resources.NewResourceFromConf(map[string]string{"first": "10"})
+	tgRes, err = resources.NewResourceFromConf(map[string]string{"vcore": "10"})
 	assert.NilError(t, err, "failed to create resource")
-	res, err = resources.NewResourceFromConf(map[string]string{"first": "1"})
+	res, err = resources.NewResourceFromConf(map[string]string{"vcore": "1"})
 	assert.NilError(t, err, "failed to create resource")
 
 	// add node to allow allocation
@@ -1420,7 +1578,6 @@ func TestTryPlaceholderAllocate(t *testing.T) {
 	err = partition.AddApplication(app)
 	assert.NilError(t, err, "app-1 should have been added to the partition")
 	// add an ask for a placeholder and allocate
-	const taskGroup = "tg-1"
 	ask := newAllocationAskTG(phID, appID1, taskGroup, res, true)
 	err = app.AddAllocationAsk(ask)
 	assert.NilError(t, err, "failed to add placeholder ask ph-1 to app")
@@ -1521,9 +1678,9 @@ func TestFailReplacePlaceholder(t *testing.T) {
 	plugin := newFakePredicatePlugin(false, map[string]int{nodeID1: -1})
 	plugins.RegisterSchedulerPlugin(plugin)
 	var tgRes, res *resources.Resource
-	tgRes, err = resources.NewResourceFromConf(map[string]string{"first": "10"})
+	tgRes, err = resources.NewResourceFromConf(map[string]string{"vcore": "10"})
 	assert.NilError(t, err, "failed to create resource")
-	res, err = resources.NewResourceFromConf(map[string]string{"first": "1"})
+	res, err = resources.NewResourceFromConf(map[string]string{"vcore": "1"})
 	assert.NilError(t, err, "failed to create resource")
 
 	// add node to allow allocation
@@ -1630,7 +1787,7 @@ func TestAddAllocationAsk(t *testing.T) {
 	assert.NilError(t, err, "app-1 should have been added to the partition")
 	// a simple ask (no repeat should fail)
 	var res *resources.Resource
-	res, err = resources.NewResourceFromConf(map[string]string{"first": "10"})
+	res, err = resources.NewResourceFromConf(map[string]string{"vcore": "10"})
 	assert.NilError(t, err, "failed to create resource")
 	askKey := "ask-key-1"
 	ask := si.AllocationAsk{
@@ -1661,7 +1818,7 @@ func TestRemoveAllocationAsk(t *testing.T) {
 	err = partition.AddApplication(app)
 	assert.NilError(t, err, "app-1 should have been added to the partition")
 	var res *resources.Resource
-	res, err = resources.NewResourceFromConf(map[string]string{"first": "10"})
+	res, err = resources.NewResourceFromConf(map[string]string{"vcore": "10"})
 	assert.NilError(t, err, "failed to create resource")
 	askKey := "ask-key-1"
 	ask := newAllocationAsk(askKey, appID1, res)
@@ -1703,3 +1860,34 @@ func TestRemoveAllocationAsk(t *testing.T) {
 	partition.removeAllocationAsk(release)
 	assert.Assert(t, resources.IsZero(app.GetPendingResource()), "app should not have pending asks")
 }
+
+func TestUpdateNodeSortingPolicy(t *testing.T) {
+	partition, err := newBasePartition()
+	assert.NilError(t, err, "partition create failed")
+
+	assert.Equal(t, partition.nodes.GetNodeSortingPolicy().PolicyType(), policies.FairnessPolicy)
+
+	partition.updateNodeSortingPolicy(configs.PartitionConfig{
+		Name: "test",
+		Queues: []configs.QueueConfig{
+			{
+				Name:      "root",
+				Parent:    true,
+				SubmitACL: "*",
+				Queues: []configs.QueueConfig{
+					{
+						Name:   "default",
+						Parent: false,
+						Queues: nil,
+					},
+				},
+			},
+		},
+		PlacementRules: nil,
+		Limits:         nil,
+		Preemption:     configs.PartitionPreemptionConfig{},
+		NodeSortPolicy: configs.NodeSortingPolicy{Type: policies.BinPackingPolicy.String()},
+	})
+
+	assert.Equal(t, partition.nodes.GetNodeSortingPolicy().PolicyType(), policies.BinPackingPolicy)
+}
