@@ -20,29 +20,20 @@ package policies
 
 import (
 	"fmt"
-
-	"go.uber.org/zap"
-
-	"github.com/apache/incubator-yunikorn-core/pkg/log"
 )
 
-type NodeSortingPolicy struct {
-	PolicyType SortingPolicy
-}
-
 type SortingPolicy int
 
 const (
 	BinPackingPolicy SortingPolicy = iota
 	FairnessPolicy
-	Unknown
 )
 
 func (nsp SortingPolicy) String() string {
-	return [...]string{"binpacking", "fair", "undefined"}[nsp]
+	return [...]string{"binpacking", "fair"}[nsp]
 }
 
-func FromString(str string) (SortingPolicy, error) {
+func SortingPolicyFromString(str string) (SortingPolicy, error) {
 	switch str {
 	// fair is the default policy when not set
 	case FairnessPolicy.String(), "":
@@ -50,21 +41,7 @@ func FromString(str string) (SortingPolicy, error) {
 	case BinPackingPolicy.String():
 		return BinPackingPolicy, nil
 	default:
-		return Unknown, fmt.Errorf("undefined policy: %s", str)
+		return FairnessPolicy, fmt.Errorf("undefined policy: %s", str)
 	}
 }
 
-func NewNodeSortingPolicy(policyType string) *NodeSortingPolicy {
-	pType, err := FromString(policyType)
-	if err != nil {
-		log.Logger().Debug("node sorting policy defaulted to 'undefined'",
-			zap.Error(err))
-	}
-	sp := &NodeSortingPolicy{
-		PolicyType: pType,
-	}
-
-	log.Logger().Debug("new node sorting policy added",
-		zap.String("type", pType.String()))
-	return sp
-}
