@@ -31,10 +31,12 @@ import (
 	"go.uber.org/zap"
 	"gopkg.in/yaml.v2"
 
+	"github.com/apache/incubator-yunikorn-core/pkg/common"
 	"github.com/apache/incubator-yunikorn-core/pkg/common/configs"
 	"github.com/apache/incubator-yunikorn-core/pkg/common/resources"
 	"github.com/apache/incubator-yunikorn-core/pkg/log"
 	metrics2 "github.com/apache/incubator-yunikorn-core/pkg/metrics"
+	"github.com/apache/incubator-yunikorn-core/pkg/metrics/history"
 	"github.com/apache/incubator-yunikorn-core/pkg/plugins"
 	"github.com/apache/incubator-yunikorn-core/pkg/scheduler"
 	"github.com/apache/incubator-yunikorn-core/pkg/scheduler/objects"
@@ -79,30 +81,16 @@ func getClusterInfo(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 
 	lists := schedulerContext.GetPartitionMapClone()
-	for _, partition := range lists {
-		clusterInfo := getClusterJSON(partition)
-		var clustersInfo []dao.ClusterDAOInfo
-		clustersInfo = append(clustersInfo, *clusterInfo)
-
-		if err := json.NewEncoder(w).Encode(clustersInfo); err != nil {
-			buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
-		}
+	clustersInfo := getClusterDAO(lists)
+	if err := json.NewEncoder(w).Encode(clustersInfo); err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
 }
 
 func getClusterUtilization(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
-	var clusterUtil []*dao.ClustersUtilDAOInfo
-	var utilizations []*dao.ClusterUtilDAOInfo
 	lists := schedulerContext.GetPartitionMapClone()
-	for _, partition := range lists {
-		utilizations = getClusterUtilJSON(partition)
-		clusterUtil = append(clusterUtil, &dao.ClustersUtilDAOInfo{
-			PartitionName: partition.Name,
-			ClustersUtil:  utilizations,
-		})
-	}
-
+	clusterUtil := getClustersUtilDAO(lists)
 	if err := json.NewEncoder(w).Encode(clusterUtil); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
@@ -111,20 +99,20 @@ func getClusterUtilization(w http.ResponseWriter, r *http.Request) {
 func getApplicationsInfo(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 
-	queueName := r.URL.Query().Get("queue")
-	queueErr := validateQueue(queueName)
+	queuePath := r.URL.Query().Get("queue")
+	queueErr := validateQueue(queuePath)
 	if queueErr != nil {
 		buildJSONErrorResponse(w, queueErr.Error(), http.StatusBadRequest)
 		return
 	}
 
-	var appsDao []*dao.ApplicationDAOInfo
 	lists := schedulerContext.GetPartitionMapClone()
+	appsDao := make([]*dao.ApplicationDAOInfo, 0, len(lists))
 	for _, partition := range lists {
 		appList := partition.GetApplications()
 		appList = append(appList, partition.GetCompletedApplications()...)
 		for _, app := range appList {
-			if len(queueName) == 0 || strings.EqualFold(queueName, app.GetQueueName()) {
+			if len(queuePath) == 0 || strings.EqualFold(queuePath, app.GetQueuePath()) {
 				appsDao = append(appsDao, getApplicationJSON(app))
 			}
 		}
@@ -135,14 +123,14 @@ func getApplicationsInfo(w http.ResponseWriter, r *http.Request) {
 	}
 }
 
-func validateQueue(queueName string) error {
-	if queueName != "" {
-		queueNameArr := strings.Split(queueName, ".")
+func validateQueue(queuePath string) error {
+	if queuePath != "" {
+		queueNameArr := strings.Split(queuePath, ".")
 		for _, name := range queueNameArr {
 			if !configs.QueueNameRegExp.MatchString(name) {
 				return fmt.Errorf("problem in queue query parameter parsing as queue param "+
 					"%s contains invalid queue name %s. Queue name must only have "+
-					"alphanumeric characters, - or _, and be no longer than 64 characters", queueName, name)
+					"alphanumeric characters, - or _, and be no longer than 64 characters", queuePath, name)
 			}
 		}
 	}
@@ -152,20 +140,8 @@ func validateQueue(queueName string) error {
 func getNodesInfo(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 
-	var result []*dao.NodesDAOInfo
 	lists := schedulerContext.GetPartitionMapClone()
-	for _, partition := range lists {
-		var nodesDao []*dao.NodeDAOInfo
-		for _, node := range partition.GetNodes() {
-			nodeDao := getNodeJSON(node)
-			nodesDao = append(nodesDao, nodeDao)
-		}
-		result = append(result, &dao.NodesDAOInfo{
-			PartitionName: partition.Name,
-			Nodes:         nodesDao,
-		})
-	}
-
+	result := getNodesDAO(lists)
 	if err := json.NewEncoder(w).Encode(result); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
@@ -175,13 +151,7 @@ func getNodesUtilization(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 
 	lists := schedulerContext.GetPartitionMapClone()
-	var result []*dao.NodesUtilDAOInfo
-	for _, partition := range lists {
-		for name := range partition.GetTotalPartitionResource().Resources {
-			nodesUtil := getNodesUtilJSON(partition, name)
-			result = append(result, nodesUtil)
-		}
-	}
+	result := getNodesUtilDAO(lists)
 	if err := json.NewEncoder(w).Encode(result); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
@@ -223,6 +193,7 @@ func buildJSONErrorResponse(w http.ResponseWriter, detail string, code int) {
 
 func getClusterJSON(partition *scheduler.PartitionContext) *dao.ClusterDAOInfo {
 	clusterInfo := &dao.ClusterDAOInfo{}
+	clusterInfo.PartitionName = common.GetPartitionNameWithoutClusterID(partition.Name)
 	clusterInfo.TotalApplications = strconv.Itoa(partition.GetTotalApplicationCount())
 	clusterInfo.TotalContainers = strconv.Itoa(partition.GetTotalAllocationCount())
 	clusterInfo.TotalNodes = strconv.Itoa(partition.GetTotalNodeCount())
@@ -274,7 +245,7 @@ func getPartitionJSON(partition *scheduler.PartitionContext) *dao.PartitionDAOIn
 
 	queueDAOInfo := partition.GetQueueInfos()
 
-	partitionInfo.PartitionName = partition.Name
+	partitionInfo.PartitionName = common.GetPartitionNameWithoutClusterID(partition.Name)
 	partitionInfo.Capacity = dao.PartitionCapacity{
 		Capacity:     partition.GetTotalPartitionResource().DAOString(),
 		UsedCapacity: partition.GetAllocatedResource().DAOString(),
@@ -285,8 +256,8 @@ func getPartitionJSON(partition *scheduler.PartitionContext) *dao.PartitionDAOIn
 }
 
 func getApplicationJSON(app *objects.Application) *dao.ApplicationDAOInfo {
-	var allocationInfos []dao.AllocationDAOInfo
 	allocations := app.GetAllAllocations()
+	allocationInfos := make([]dao.AllocationDAOInfo, 0, len(allocations))
 	for _, alloc := range allocations {
 		allocInfo := dao.AllocationDAOInfo{
 			AllocationKey:    alloc.AllocationKey,
@@ -305,8 +276,8 @@ func getApplicationJSON(app *objects.Application) *dao.ApplicationDAOInfo {
 	return &dao.ApplicationDAOInfo{
 		ApplicationID:  app.ApplicationID,
 		UsedResource:   app.GetAllocatedResource().DAOString(),
-		Partition:      app.Partition,
-		QueueName:      app.QueueName,
+		Partition:      common.GetPartitionNameWithoutClusterID(app.Partition),
+		QueueName:      app.QueuePath,
 		SubmissionTime: app.SubmissionTime.UnixNano(),
 		Allocations:    allocationInfos,
 		State:          app.CurrentState(),
@@ -314,8 +285,9 @@ func getApplicationJSON(app *objects.Application) *dao.ApplicationDAOInfo {
 }
 
 func getNodeJSON(node *objects.Node) *dao.NodeDAOInfo {
-	var allocations []*dao.AllocationDAOInfo
-	for _, alloc := range node.GetAllAllocations() {
+	apps := node.GetAllAllocations()
+	allocations := make([]*dao.AllocationDAOInfo, 0, len(apps))
+	for _, alloc := range apps {
 		allocInfo := &dao.AllocationDAOInfo{
 			AllocationKey:    alloc.AllocationKey,
 			AllocationTags:   alloc.Tags,
@@ -347,9 +319,10 @@ func getNodesUtilJSON(partition *scheduler.PartitionContext, name string) *dao.N
 	mapResult := make([]int, 10)
 	mapName := make([][]string, 10)
 	var v float64
-	var resourceExist = true
 	var nodeUtil []*dao.NodeUtilDAOInfo
 	for _, node := range partition.GetNodes() {
+		resourceExist := true
+		// check resource exist or not
 		total := node.GetCapacity()
 		if total.Resources[name] <= 0 {
 			resourceExist = false
@@ -358,6 +331,7 @@ func getNodesUtilJSON(partition *scheduler.PartitionContext, name string) *dao.N
 		if _, ok := resourceAllocated.Resources[name]; !ok {
 			resourceExist = false
 		}
+		// if resource exist in node, record the bucket it should go
 		if resourceExist {
 			v = float64(resources.CalculateAbsUsedCapacity(total, resourceAllocated).Resources[name])
 			idx := int(math.Dim(math.Ceil(v/10), 1))
@@ -365,24 +339,14 @@ func getNodesUtilJSON(partition *scheduler.PartitionContext, name string) *dao.N
 			mapName[idx] = append(mapName[idx], node.NodeID)
 		}
 	}
+	// put number of nodes and node name to different buckets
 	for k := 0; k < 10; k++ {
-		if resourceExist {
-			util := &dao.NodeUtilDAOInfo{
-				BucketName: fmt.Sprintf("%d", k*10) + "-" + fmt.Sprintf("%d", (k+1)*10) + "%",
-				NumOfNodes: int64(mapResult[k]),
-				NodeNames:  mapName[k],
-			}
-			nodeUtil = append(nodeUtil, util)
-		} else {
-			util := &dao.NodeUtilDAOInfo{
-				BucketName: fmt.Sprintf("%d", k*10) + "-" + fmt.Sprintf("%d", (k+1)*10) + "%",
-				NumOfNodes: int64(-1),
-				NodeNames:  []string{"N/A"},
-			}
-			nodeUtil = append(nodeUtil, util)
+		util := &dao.NodeUtilDAOInfo{
+			BucketName: fmt.Sprintf("%d", k*10) + "-" + fmt.Sprintf("%d", (k+1)*10) + "%",
+			NumOfNodes: int64(mapResult[k]),
+			NodeNames:  mapName[k],
 		}
-		mapResult[k] = 0
-		mapName[k] = []string{}
+		nodeUtil = append(nodeUtil, util)
 	}
 	return &dao.NodesUtilDAOInfo{
 		ResourceType: name,
@@ -398,20 +362,10 @@ func getApplicationHistory(w http.ResponseWriter, r *http.Request) {
 		buildJSONErrorResponse(w, "Internal metrics collection is not enabled.", http.StatusNotImplemented)
 		return
 	}
-	var result []*dao.ApplicationHistoryDAOInfo
 	// get a copy of the records: if the array contains nil values they will always be at the
 	// start and we cannot shortcut the loop using a break, we must finish iterating
 	records := imHistory.GetRecords()
-	for _, record := range records {
-		if record == nil {
-			continue
-		}
-		element := &dao.ApplicationHistoryDAOInfo{
-			Timestamp:         record.Timestamp.UnixNano(),
-			TotalApplications: strconv.Itoa(record.TotalApplications),
-		}
-		result = append(result, element)
-	}
+	result := getAppHistoryDAO(records)
 	if err := json.NewEncoder(w).Encode(result); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
@@ -425,20 +379,10 @@ func getContainerHistory(w http.ResponseWriter, r *http.Request) {
 		buildJSONErrorResponse(w, "Internal metrics collection is not enabled.", http.StatusNotImplemented)
 		return
 	}
-	var result []*dao.ContainerHistoryDAOInfo
 	// get a copy of the records: if the array contains nil values they will always be at the
 	// start and we cannot shortcut the loop using a break, we must finish iterating
 	records := imHistory.GetRecords()
-	for _, record := range records {
-		if record == nil {
-			continue
-		}
-		element := &dao.ContainerHistoryDAOInfo{
-			Timestamp:       record.Timestamp.UnixNano(),
-			TotalContainers: strconv.Itoa(record.TotalContainers),
-		}
-		result = append(result, element)
-	}
+	result := getContainerHistoryDAO(records)
 	if err := json.NewEncoder(w).Encode(result); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
@@ -547,6 +491,10 @@ func checkHealthStatus(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 	metrics := metrics2.GetSchedulerMetrics()
 	result := scheduler.GetSchedulerHealthStatus(metrics, schedulerContext)
+	if !result.Healthy {
+		log.Logger().Error("Scheduler is not healthy", zap.Any("health check values", result.HealthChecks))
+		buildJSONErrorResponse(w, "Scheduler is not healthy", http.StatusServiceUnavailable)
+	}
 	if err := json.NewEncoder(w).Encode(result); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
@@ -566,7 +514,7 @@ func buildUpdateResponse(err error, w http.ResponseWriter) {
 }
 
 func updateConfiguration(conf string) (string, error) {
-	if plugin := plugins.GetConfigPlugin(); plugin != nil {
+	if plugin := plugins.GetResourceManagerCallbackPlugin(); plugin != nil {
 		// use the plugin to update the configuration in the configMap
 		resp := plugin.UpdateConfiguration(&si.UpdateConfigurationRequest{
 			Configs: conf,
@@ -582,32 +530,8 @@ func updateConfiguration(conf string) (string, error) {
 func getPartitions(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 
-	var partitionsInfo []*dao.PartitionInfo
 	lists := schedulerContext.GetPartitionMapClone()
-	for _, partitionContext := range lists {
-		partitionInfo := &dao.PartitionInfo{}
-		partitionInfo.Name = partitionContext.Name
-		partitionInfo.State = partitionContext.GetCurrentState()
-		partitionInfo.LastStateTransitionTime = partitionContext.GetStateTime().String()
-
-		capacityInfo := dao.PartitionCapacity{}
-		capacityInfo.Capacity = partitionContext.GetTotalPartitionResource().DAOString()
-		capacityInfo.UsedCapacity = partitionContext.GetAllocatedResource().DAOString()
-		partitionInfo.Capacity = capacityInfo
-		partitionInfo.NodeSortingPolicy = partitionContext.GetNodeSortingPolicy().String()
-
-		appList := partitionContext.GetApplications()
-		appList = append(appList, partitionContext.GetCompletedApplications()...)
-		applicationsState := make(map[string]int)
-		totalApplications := 0
-		for _, app := range appList {
-			applicationsState[app.CurrentState()]++
-			totalApplications++
-		}
-		applicationsState["total"] = totalApplications
-		partitionInfo.Applications = applicationsState
-		partitionsInfo = append(partitionsInfo, partitionInfo)
-	}
+	partitionsInfo := getPartitionInfoDAO(lists)
 	if err := json.NewEncoder(w).Encode(partitionsInfo); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
@@ -652,8 +576,9 @@ func getPartitionNodes(w http.ResponseWriter, r *http.Request) {
 	}
 	partitionContext := schedulerContext.GetPartitionWithoutClusterID(partition)
 	if partitionContext != nil {
-		var nodesDao []*dao.NodeDAOInfo
-		for _, node := range partitionContext.GetNodes() {
+		ns := partitionContext.GetNodes()
+		nodesDao := make([]*dao.NodeDAOInfo, 0, len(ns))
+		for _, node := range ns {
 			nodeDao := getNodeJSON(node)
 			nodesDao = append(nodesDao, nodeDao)
 		}
@@ -687,29 +612,200 @@ func getQueueApplications(w http.ResponseWriter, r *http.Request) {
 		buildJSONErrorResponse(w, "Incorrect URL path. Please check the usage documentation", http.StatusBadRequest)
 		return
 	}
-	var appExistsInPartition = false
-	var appExistsInQueue = false
-	var appsDao []*dao.ApplicationDAOInfo
+
 	partitionContext := schedulerContext.GetPartitionWithoutClusterID(partition)
-	if partitionContext != nil {
-		appExistsInPartition = true
-		appList := partitionContext.GetApplications()
-		for _, app := range appList {
-			if strings.EqualFold(queueName, app.GetQueueName()) {
-				appExistsInQueue = true
-				appsDao = append(appsDao, getApplicationJSON(app))
-			}
-		}
-	}
-	if !appExistsInPartition {
+	if partitionContext == nil {
 		buildJSONErrorResponse(w, "Partition not found", http.StatusBadRequest)
 		return
 	}
-	if !appExistsInQueue {
+
+	queue := partitionContext.GetQueue(queueName)
+	if queue == nil {
 		buildJSONErrorResponse(w, "Queue not found", http.StatusBadRequest)
 		return
 	}
+
+	apps := queue.GetCopyOfApps()
+	completedApps := queue.GetCopyOfCompletedApps()
+	appsDao := make([]*dao.ApplicationDAOInfo, 0, len(apps)+len(completedApps))
+	for _, app := range apps {
+		appsDao = append(appsDao, getApplicationJSON(app))
+	}
+	for _, app := range completedApps {
+		appsDao = append(appsDao, getApplicationJSON(app))
+	}
+
 	if err := json.NewEncoder(w).Encode(appsDao); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
 }
+
+func setLogLevel(w http.ResponseWriter, r *http.Request) {
+	vars := mux.Vars(r)
+	writeHeaders(w)
+	level := vars["level"]
+	if err := log.SetLogLevel(level); err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
+	}
+}
+
+func getLogLevel(w http.ResponseWriter, r *http.Request) {
+	writeHeaders(w)
+	zapConfig := log.GetConfig()
+	if _, err := w.Write([]byte(zapConfig.Level.Level().String())); err != nil {
+		log.Logger().Error("Could not get log level", zap.Error(err))
+		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
+	}
+}
+
+func getPartitionInfoDAO(lists map[string]*scheduler.PartitionContext) []*dao.PartitionInfo {
+	var result []*dao.PartitionInfo
+
+	for _, partitionContext := range lists {
+		partitionInfo := &dao.PartitionInfo{}
+		partitionInfo.ClusterID = partitionContext.RmID
+		partitionInfo.Name = common.GetPartitionNameWithoutClusterID(partitionContext.Name)
+		partitionInfo.State = partitionContext.GetCurrentState()
+		partitionInfo.LastStateTransitionTime = partitionContext.GetStateTime().String()
+
+		capacityInfo := dao.PartitionCapacity{}
+		capacityInfo.Capacity = partitionContext.GetTotalPartitionResource().DAOString()
+		capacityInfo.UsedCapacity = partitionContext.GetAllocatedResource().DAOString()
+		partitionInfo.Capacity = capacityInfo
+		partitionInfo.NodeSortingPolicy = dao.NodeSortingPolicy{
+			Type:            partitionContext.GetNodeSortingPolicyType().String(),
+			ResourceWeights: partitionContext.GetNodeSortingResourceWeights(),
+		}
+
+		appList := partitionContext.GetApplications()
+		appList = append(appList, partitionContext.GetCompletedApplications()...)
+		applicationsState := make(map[string]int)
+		totalApplications := 0
+		for _, app := range appList {
+			applicationsState[app.CurrentState()]++
+			totalApplications++
+		}
+		applicationsState["total"] = totalApplications
+		partitionInfo.Applications = applicationsState
+		result = append(result, partitionInfo)
+	}
+
+	return result
+}
+
+func getAppHistoryDAO(records []*history.MetricsRecord) []*dao.ApplicationHistoryDAOInfo {
+	var result []*dao.ApplicationHistoryDAOInfo
+
+	for _, record := range records {
+		if record == nil {
+			continue
+		}
+		element := &dao.ApplicationHistoryDAOInfo{
+			Timestamp:         record.Timestamp.UnixNano(),
+			TotalApplications: strconv.Itoa(record.TotalApplications),
+		}
+		result = append(result, element)
+	}
+
+	return result
+}
+
+func getNodesDAO(lists map[string]*scheduler.PartitionContext) []*dao.NodesDAOInfo {
+	var result []*dao.NodesDAOInfo
+
+	for _, partition := range lists {
+		ns := partition.GetNodes()
+		nodesDao := make([]*dao.NodeDAOInfo, 0, len(ns))
+		for _, node := range ns {
+			nodeDao := getNodeJSON(node)
+			nodesDao = append(nodesDao, nodeDao)
+		}
+		result = append(result, &dao.NodesDAOInfo{
+			PartitionName: common.GetPartitionNameWithoutClusterID(partition.Name),
+			Nodes:         nodesDao,
+		})
+	}
+
+	return result
+}
+
+func getContainerHistoryDAO(records []*history.MetricsRecord) []*dao.ContainerHistoryDAOInfo {
+	var result []*dao.ContainerHistoryDAOInfo
+
+	for _, record := range records {
+		if record == nil {
+			continue
+		}
+		element := &dao.ContainerHistoryDAOInfo{
+			Timestamp:       record.Timestamp.UnixNano(),
+			TotalContainers: strconv.Itoa(record.TotalContainers),
+		}
+		result = append(result, element)
+	}
+
+	return result
+}
+
+func getClustersUtilDAO(lists map[string]*scheduler.PartitionContext) []*dao.ClustersUtilDAOInfo {
+	var result []*dao.ClustersUtilDAOInfo
+
+	for _, partition := range lists {
+		result = append(result, &dao.ClustersUtilDAOInfo{
+			PartitionName: common.GetPartitionNameWithoutClusterID(partition.Name),
+			ClustersUtil:  getClusterUtilJSON(partition),
+		})
+	}
+
+	return result
+}
+func getNodesUtilDAO(lists map[string]*scheduler.PartitionContext) []*dao.NodesUtilDAOInfo {
+	var result []*dao.NodesUtilDAOInfo
+
+	for _, partition := range lists {
+		partitionResource := partition.GetTotalPartitionResource()
+		// partitionResource can be null if the partition has no node
+		if partitionResource != nil {
+			for name := range partitionResource.Resources {
+				result = append(result, getNodesUtilJSON(partition, name))
+			}
+		}
+	}
+
+	return result
+}
+
+func getApplicationsDAO(lists map[string]*scheduler.PartitionContext) []*dao.ApplicationDAOInfo {
+	var result []*dao.ApplicationDAOInfo
+
+	for _, partition := range lists {
+		var appList []*objects.Application
+		appList = append(appList, partition.GetApplications()...)
+		appList = append(appList, partition.GetCompletedApplications()...)
+
+		for _, app := range appList {
+			result = append(result, getApplicationJSON(app))
+		}
+	}
+
+	return result
+}
+
+func getPartitionDAO(lists map[string]*scheduler.PartitionContext) []*dao.PartitionDAOInfo {
+	var result []*dao.PartitionDAOInfo
+
+	for _, partition := range lists {
+		result = append(result, getPartitionJSON(partition))
+	}
+
+	return result
+}
+
+func getClusterDAO(lists map[string]*scheduler.PartitionContext) []*dao.ClusterDAOInfo {
+	var result []*dao.ClusterDAOInfo
+
+	for _, partition := range lists {
+		result = append(result, getClusterJSON(partition))
+	}
+
+	return result
+}
