@@ -22,11 +22,14 @@ import (
 	"crypto/sha256"
 	"encoding/json"
 	"fmt"
+	"io/ioutil"
 	"net/http"
 	"net/http/httptest"
+	"os"
 	"reflect"
 	"strings"
 	"testing"
+	"time"
 
 	"github.com/gorilla/mux"
 	"github.com/prometheus/client_golang/prometheus/promhttp"
@@ -46,6 +49,7 @@ import (
 )
 
 const partitionNameWithoutClusterID = "default"
+const normalizedPartitionName = "[rm-123]default"
 const startConf = `
 partitions:
   - name: default
@@ -94,6 +98,7 @@ partitions:
         submitacl: "*"
         queues:
           - name: default
+          - name: noapps
 `
 
 const configMultiPartitions = `
@@ -129,6 +134,16 @@ partitions:
     queues: 
       - 
         name: root
+        properties:
+          application.sort.policy: stateaware
+        childtemplate:
+          properties:
+            application.sort.policy: stateaware
+          resources:
+            guaranteed:
+              memory: 400000
+            max:
+              memory: 600000
         queues: 
           - 
             name: a
@@ -469,6 +484,39 @@ type FakeConfigPlugin struct {
 	generateError bool
 }
 
+func (f *FakeConfigPlugin) UpdateAllocation(response *si.AllocationResponse) error {
+	// do nothing
+	return nil
+}
+
+func (f *FakeConfigPlugin) UpdateApplication(response *si.ApplicationResponse) error {
+	// do nothing
+	return nil
+}
+
+func (f *FakeConfigPlugin) UpdateNode(response *si.NodeResponse) error {
+	// do nothing
+	return nil
+}
+
+func (f *FakeConfigPlugin) Predicates(args *si.PredicatesArgs) error {
+	// do nothing
+	return nil
+}
+
+func (f *FakeConfigPlugin) ReSyncSchedulerCache(args *si.ReSyncSchedulerCacheArgs) error {
+	// do nothing
+	return nil
+}
+
+func (f *FakeConfigPlugin) SendEvent(events []*si.EventRecord) {
+	// do nothing
+}
+
+func (f *FakeConfigPlugin) UpdateContainerSchedulingState(request *si.UpdateContainerSchedulingStateRequest) {
+	// do nothing
+}
+
 func (f FakeConfigPlugin) UpdateConfiguration(args *si.UpdateConfigurationRequest) *si.UpdateConfigurationResponse {
 	if f.generateError {
 		return &si.UpdateConfigurationResponse{
@@ -599,7 +647,7 @@ func TestGetClusterUtilJSON(t *testing.T) {
 
 	// add node to partition with allocations
 	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{resources.MEMORY: 1000, resources.VCORE: 1000}).ToProto()
-	node1 := objects.NewNode(&si.NewNodeInfo{NodeID: nodeID, SchedulableResource: nodeRes})
+	node1 := objects.NewNode(&si.NodeInfo{NodeID: nodeID, SchedulableResource: nodeRes})
 
 	resAlloc1 := resources.NewResourceFromMap(map[string]resources.Quantity{resources.MEMORY: 500, resources.VCORE: 300})
 	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{resources.MEMORY: 300, resources.VCORE: 200})
@@ -658,15 +706,14 @@ func ContainsObj(slice interface{}, contains interface{}) bool {
 
 func TestGetNodesUtilJSON(t *testing.T) {
 	configs.MockSchedulerConfigByData([]byte(configDefault))
-	var err error
-	schedulerContext, err = scheduler.NewClusterContext(rmID, policyGroup)
+	context, err := scheduler.NewClusterContext(rmID, policyGroup)
 	assert.NilError(t, err, "Error when load schedulerContext from config")
-	assert.Equal(t, 1, len(schedulerContext.GetPartitionMapClone()))
-
+	assert.Equal(t, 1, len(context.GetPartitionMapClone()))
 	// Check test partition
 	partitionName := common.GetNormalizedPartitionName("default", rmID)
-	partition := schedulerContext.GetPartition(partitionName)
+	partition := context.GetPartition(partitionName)
 	assert.Equal(t, partitionName, partition.Name)
+
 	// create test application
 	appID := "app1"
 	app := newApplication(appID, partitionName, queueName, rmID)
@@ -675,13 +722,15 @@ func TestGetNodesUtilJSON(t *testing.T) {
 
 	// create test nodes
 	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{resources.MEMORY: 1000, resources.VCORE: 1000}).ToProto()
+	nodeRes2 := resources.NewResourceFromMap(map[string]resources.Quantity{resources.MEMORY: 1000, resources.VCORE: 1000, "GPU": 10}).ToProto()
 	node1ID := "node-1"
-	node1 := objects.NewNode(&si.NewNodeInfo{NodeID: node1ID, SchedulableResource: nodeRes})
+	node1 := objects.NewNode(&si.NodeInfo{NodeID: node1ID, SchedulableResource: nodeRes})
 	node2ID := "node-2"
-	node2 := objects.NewNode(&si.NewNodeInfo{NodeID: node2ID, SchedulableResource: nodeRes})
+	node2 := objects.NewNode(&si.NodeInfo{NodeID: node2ID, SchedulableResource: nodeRes2})
+
 	// create test allocations
 	resAlloc1 := resources.NewResourceFromMap(map[string]resources.Quantity{resources.MEMORY: 500, resources.VCORE: 300})
-	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{resources.MEMORY: 300, resources.VCORE: 500})
+	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{resources.MEMORY: 300, resources.VCORE: 500, "GPU": 5})
 	ask1 := &objects.AllocationAsk{
 		AllocationKey:     "alloc-1",
 		QueueName:         queueName,
@@ -704,9 +753,11 @@ func TestGetNodesUtilJSON(t *testing.T) {
 	// get nodes utilization
 	res1 := getNodesUtilJSON(partition, resources.MEMORY)
 	res2 := getNodesUtilJSON(partition, resources.VCORE)
+	res3 := getNodesUtilJSON(partition, "GPU")
 	resNon := getNodesUtilJSON(partition, "non-exist")
 	subres1 := res1.NodesUtil
 	subres2 := res2.NodesUtil
+	subres3 := res3.NodesUtil
 	subresNon := resNon.NodesUtil
 
 	assert.Equal(t, res1.ResourceType, resources.MEMORY)
@@ -721,9 +772,13 @@ func TestGetNodesUtilJSON(t *testing.T) {
 	assert.Equal(t, subres2[2].NodeNames[0], node1ID)
 	assert.Equal(t, subres2[4].NodeNames[0], node2ID)
 
+	assert.Equal(t, res3.ResourceType, "GPU")
+	assert.Equal(t, subres3[4].NumOfNodes, int64(1))
+	assert.Equal(t, subres3[4].NodeNames[0], node2ID)
+
 	assert.Equal(t, resNon.ResourceType, "non-exist")
-	assert.Equal(t, subresNon[0].NumOfNodes, int64(-1))
-	assert.Equal(t, subresNon[0].NodeNames[0], "N/A")
+	assert.Equal(t, subresNon[0].NumOfNodes, int64(0))
+	assert.Equal(t, len(subresNon[0].NodeNames), 0)
 }
 
 func addAndConfirmApplicationExists(t *testing.T, partitionName string, partition *scheduler.PartitionContext, appName string) *objects.Application {
@@ -784,30 +839,40 @@ func TestPartitions(t *testing.T) {
 	req, err = http.NewRequest("GET", "/ws/v1/partitions", strings.NewReader(""))
 	assert.NilError(t, err, "App Handler request failed")
 	resp := &MockResponseWriter{}
-	var partitionInfo []dao.PartitionInfo
+	var partitionInfo []*dao.PartitionInfo
 	getPartitions(resp, req)
 	err = json.Unmarshal(resp.outputBytes, &partitionInfo)
 	assert.NilError(t, err, "failed to unmarshal applications dao response from response body: %s", string(resp.outputBytes))
-	for _, part := range partitionInfo {
-		if part.Name == partitionName {
-			assert.Equal(t, part.Name, "[rm-123]default")
-			assert.Equal(t, part.NodeSortingPolicy, "fair")
-			assert.Equal(t, part.Applications["total"], 8)
-			assert.Equal(t, part.Applications[objects.New.String()], 1)
-			assert.Equal(t, part.Applications[objects.Accepted.String()], 1)
-			assert.Equal(t, part.Applications[objects.Starting.String()], 1)
-			assert.Equal(t, part.Applications[objects.Running.String()], 1)
-			assert.Equal(t, part.Applications[objects.Completing.String()], 1)
-			assert.Equal(t, part.Applications[objects.Rejected.String()], 1)
-			assert.Equal(t, part.Applications[objects.Completed.String()], 1)
-			assert.Equal(t, part.Applications[objects.Failed.String()], 1)
-			assert.Equal(t, part.State, "Active")
-		} else {
-			assert.Equal(t, part.Name, "[rm-123]gpu")
-			assert.Equal(t, part.NodeSortingPolicy, "fair")
-			assert.Equal(t, part.Applications["total"], 0)
-		}
+
+	cs := make(map[string]*dao.PartitionInfo, 2)
+	for _, d := range partitionInfo {
+		cs[d.Name] = d
 	}
+
+	assert.Assert(t, cs["default"] != nil)
+	assert.Equal(t, cs["default"].ClusterID, "rm-123")
+	assert.Equal(t, cs["default"].Name, "default")
+	assert.Equal(t, cs["default"].NodeSortingPolicy.Type, "fair")
+	assert.Equal(t, cs["default"].NodeSortingPolicy.ResourceWeights["vcore"], 1.0)
+	assert.Equal(t, cs["default"].NodeSortingPolicy.ResourceWeights["memory"], 1.0)
+	assert.Equal(t, cs["default"].Applications["total"], 8)
+	assert.Equal(t, cs["default"].Applications[objects.New.String()], 1)
+	assert.Equal(t, cs["default"].Applications[objects.Accepted.String()], 1)
+	assert.Equal(t, cs["default"].Applications[objects.Starting.String()], 1)
+	assert.Equal(t, cs["default"].Applications[objects.Running.String()], 1)
+	assert.Equal(t, cs["default"].Applications[objects.Completing.String()], 1)
+	assert.Equal(t, cs["default"].Applications[objects.Rejected.String()], 1)
+	assert.Equal(t, cs["default"].Applications[objects.Completed.String()], 1)
+	assert.Equal(t, cs["default"].Applications[objects.Failed.String()], 1)
+	assert.Equal(t, cs["default"].State, "Active")
+
+	assert.Assert(t, cs["gpu"] != nil)
+	assert.Equal(t, cs["gpu"].ClusterID, "rm-123")
+	assert.Equal(t, cs["gpu"].Name, "gpu")
+	assert.Equal(t, cs["default"].NodeSortingPolicy.Type, "fair")
+	assert.Equal(t, cs["default"].NodeSortingPolicy.ResourceWeights["vcore"], 1.0)
+	assert.Equal(t, cs["default"].NodeSortingPolicy.ResourceWeights["memory"], 1.0)
+	assert.Equal(t, cs["gpu"].Applications["total"], 0)
 }
 
 func TestCreateClusterConfig(t *testing.T) {
@@ -921,6 +986,22 @@ func TestGetPartitionQueuesHandler(t *testing.T) {
 	assert.Equal(t, partitionQueuesDao.Children[0].Parent, "root")
 	assert.Equal(t, partitionQueuesDao.Children[1].Parent, "root")
 	assert.Equal(t, partitionQueuesDao.Children[2].Parent, "root")
+	assert.Equal(t, len(partitionQueuesDao.Properties), 1)
+	assert.Equal(t, partitionQueuesDao.Properties["application.sort.policy"], "stateaware")
+	assert.Equal(t, len(partitionQueuesDao.TemplateInfo.Properties), 1)
+	assert.Equal(t, partitionQueuesDao.TemplateInfo.Properties["application.sort.policy"], "stateaware")
+
+	maxResourcesConf := make(map[string]string)
+	maxResourcesConf["memory"] = "600000"
+	maxResource, err := resources.NewResourceFromConf(maxResourcesConf)
+	assert.NilError(t, err)
+	assert.Equal(t, partitionQueuesDao.TemplateInfo.MaxResource, maxResource.DAOString())
+
+	guaranteedResourcesConf := make(map[string]string)
+	guaranteedResourcesConf["memory"] = "400000"
+	guaranteedResources, err := resources.NewResourceFromConf(guaranteedResourcesConf)
+	assert.NilError(t, err)
+	assert.Equal(t, partitionQueuesDao.TemplateInfo.GuaranteedResource, guaranteedResources.DAOString())
 
 	// Partition not sent as part of request
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/queues", strings.NewReader(""))
@@ -948,6 +1029,73 @@ func TestGetPartitionQueuesHandler(t *testing.T) {
 	assertPartitionExists(t, resp)
 }
 
+func TestGetClusterInfo(t *testing.T) {
+	configs.MockSchedulerConfigByData([]byte(configTwoLevelQueues))
+	var err error
+	schedulerContext, err = scheduler.NewClusterContext(rmID, policyGroup)
+	assert.NilError(t, err)
+	assert.Equal(t, 2, len(schedulerContext.GetPartitionMapClone()))
+
+	resp := &MockResponseWriter{}
+	getClusterInfo(resp, nil)
+	var data []*dao.ClusterDAOInfo
+	err = json.Unmarshal(resp.outputBytes, &data)
+	assert.NilError(t, err)
+	assert.Equal(t, 2, len(data))
+
+	cs := make(map[string]*dao.ClusterDAOInfo, 2)
+	for _, d := range data {
+		cs[d.PartitionName] = d
+	}
+
+	assert.Assert(t, cs["default"] != nil)
+	assert.Assert(t, cs["gpu"] != nil)
+}
+
+func TestGetClusterUtilization(t *testing.T) {
+	configs.MockSchedulerConfigByData([]byte(configTwoLevelQueues))
+	var err error
+	schedulerContext, err = scheduler.NewClusterContext(rmID, policyGroup)
+	assert.NilError(t, err)
+	assert.Equal(t, 2, len(schedulerContext.GetPartitionMapClone()))
+
+	resp := &MockResponseWriter{}
+	getClusterUtilization(resp, nil)
+	var data []*dao.ClustersUtilDAOInfo
+	err = json.Unmarshal(resp.outputBytes, &data)
+	assert.NilError(t, err)
+
+	cs := make(map[string]*dao.ClustersUtilDAOInfo, 2)
+	for _, d := range data {
+		cs[d.PartitionName] = d
+	}
+
+	assert.Assert(t, cs["default"] != nil)
+	assert.Assert(t, cs["gpu"] != nil)
+}
+
+func TestGetNodeInfo(t *testing.T) {
+	configs.MockSchedulerConfigByData([]byte(configTwoLevelQueues))
+	var err error
+	schedulerContext, err = scheduler.NewClusterContext(rmID, policyGroup)
+	assert.NilError(t, err)
+	assert.Equal(t, 2, len(schedulerContext.GetPartitionMapClone()))
+
+	resp := &MockResponseWriter{}
+	getNodesInfo(resp, nil)
+	var data []*dao.NodesDAOInfo
+	err = json.Unmarshal(resp.outputBytes, &data)
+	assert.NilError(t, err)
+
+	cs := make(map[string]*dao.NodesDAOInfo, 2)
+	for _, d := range data {
+		cs[d.PartitionName] = d
+	}
+
+	assert.Assert(t, cs["default"] != nil)
+	assert.Assert(t, cs["gpu"] != nil)
+}
+
 func TestGetPartitionNodes(t *testing.T) {
 	configs.MockSchedulerConfigByData([]byte(configDefault))
 	var err error
@@ -969,9 +1117,9 @@ func TestGetPartitionNodes(t *testing.T) {
 	// create test nodes
 	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{resources.MEMORY: 1000, resources.VCORE: 1000}).ToProto()
 	node1ID := "node-1"
-	node1 := objects.NewNode(&si.NewNodeInfo{NodeID: node1ID, SchedulableResource: nodeRes})
+	node1 := objects.NewNode(&si.NodeInfo{NodeID: node1ID, SchedulableResource: nodeRes})
 	node2ID := "node-2"
-	node2 := objects.NewNode(&si.NewNodeInfo{NodeID: node2ID, SchedulableResource: nodeRes})
+	node2 := objects.NewNode(&si.NodeInfo{NodeID: node2ID, SchedulableResource: nodeRes})
 
 	// create test allocations
 	resAlloc1 := resources.NewResourceFromMap(map[string]resources.Quantity{resources.MEMORY: 500, resources.VCORE: 300})
@@ -1048,12 +1196,22 @@ func TestGetQueueApplicationsHandler(t *testing.T) {
 	part := schedulerContext.GetPartition(partitionName)
 	assert.Equal(t, 0, len(part.GetApplications()))
 
-	// add a new app
-	app := newApplication("app-1", partitionName, "root.default", rmID)
-	err = part.AddApplication(app)
-	assert.NilError(t, err, "Failed to add Application to Partition.")
-	assert.Equal(t, app.CurrentState(), objects.New.String())
-	assert.Equal(t, 1, len(part.GetApplications()))
+	addApp := func(id string, queueName string, isCompleted bool) {
+		initSize := len(part.GetApplications())
+		app := newApplication(id, partitionName, queueName, rmID)
+		err = part.AddApplication(app)
+		assert.NilError(t, err, "Failed to add Application to Partition.")
+		assert.Equal(t, app.CurrentState(), objects.New.String())
+		assert.Equal(t, 1+initSize, len(part.GetApplications()))
+		if isCompleted {
+			// we don't test partition, so it is fine to skip to update partition
+			app.UnSetQueue()
+		}
+	}
+
+	// add two applications
+	addApp("app-1", "root.default", false)
+	addApp("app-2", "root.default", true)
 
 	NewWebApp(schedulerContext, nil)
 
@@ -1070,8 +1228,9 @@ func TestGetQueueApplicationsHandler(t *testing.T) {
 	getQueueApplications(resp, req)
 	err = json.Unmarshal(resp.outputBytes, &appsDao)
 	assert.NilError(t, err, "failed to unmarshal applications dao response from response body: %s", string(resp.outputBytes))
-	assert.Equal(t, len(appsDao), 1)
+	assert.Equal(t, len(appsDao), 2)
 
+	// test nonexistent partition
 	var req1 *http.Request
 	req1, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.default/applications", strings.NewReader(""))
 	vars1 := map[string]string{
@@ -1084,6 +1243,7 @@ func TestGetQueueApplicationsHandler(t *testing.T) {
 	getQueueApplications(resp1, req1)
 	assertPartitionExists(t, resp1)
 
+	// test nonexistent queue
 	var req2 *http.Request
 	req2, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.default/applications", strings.NewReader(""))
 	vars2 := map[string]string{
@@ -1095,6 +1255,22 @@ func TestGetQueueApplicationsHandler(t *testing.T) {
 	resp2 := &MockResponseWriter{}
 	getQueueApplications(resp2, req2)
 	assertQueueExists(t, resp2)
+
+	// test queue without applications
+	var req3 *http.Request
+	req3, err = http.NewRequest("GET", "/ws/v1/partition/default/queue/root.noapps/applications", strings.NewReader(""))
+	vars3 := map[string]string{
+		"partition": partitionNameWithoutClusterID,
+		"queue":     "root.noapps",
+	}
+	req3 = mux.SetURLVars(req3, vars3)
+	assert.NilError(t, err, "Get Queue Applications Handler request failed")
+	resp3 := &MockResponseWriter{}
+	var appsDao3 []*dao.ApplicationDAOInfo
+	getQueueApplications(resp3, req3)
+	err = json.Unmarshal(resp3.outputBytes, &appsDao3)
+	assert.NilError(t, err, "failed to unmarshal applications dao response from response body: %s", string(resp.outputBytes))
+	assert.Equal(t, len(appsDao3), 0)
 }
 
 func assertPartitionExists(t *testing.T, resp *MockResponseWriter) {
@@ -1127,3 +1303,214 @@ func TestValidateQueue(t *testing.T) {
 	err2 := validateQueue("root")
 	assert.NilError(t, err2, "Queue path is correct but stil throwing error.")
 }
+
+func TestGetNodesUtilization(t *testing.T) {
+	configs.MockSchedulerConfigByData([]byte(configMultiPartitions))
+	var err error
+	schedulerContext, err = scheduler.NewClusterContext(rmID, policyGroup)
+	assert.NilError(t, err)
+	assert.Equal(t, 2, len(schedulerContext.GetPartitionMapClone()))
+
+	var req *http.Request
+	req, err = http.NewRequest("GET", "/ws/v1/nodes/utilization", strings.NewReader(""))
+	req = mux.SetURLVars(req, make(map[string]string))
+	assert.NilError(t, err)
+	resp := &MockResponseWriter{}
+	var nodesDao []*dao.NodesUtilDAOInfo
+	// all partitions have no nodes, but it should be fine to get utilization
+	getNodesUtilization(resp, req)
+	err = json.Unmarshal(resp.outputBytes, &nodesDao)
+	assert.NilError(t, err)
+	assert.Equal(t, len(nodesDao), 0)
+}
+
+func TestGetFullStateDump(t *testing.T) {
+	schedulerContext = prepareSchedulerContext(t)
+
+	partitionContext := schedulerContext.GetPartitionMapClone()
+	context := partitionContext[normalizedPartitionName]
+	app := newApplication("appID", normalizedPartitionName, "root.default", rmID)
+	err := context.AddApplication(app)
+	assert.NilError(t, err, "failed to add Application to partition")
+
+	imHistory = history.NewInternalMetricsHistory(5)
+	req, err2 := http.NewRequest("GET", "/ws/v1/getfullstatedump", strings.NewReader(""))
+	assert.NilError(t, err2)
+	req = mux.SetURLVars(req, make(map[string]string))
+	resp := &MockResponseWriter{}
+
+	getFullStateDump(resp, req)
+	receivedBytes := resp.outputBytes
+	statusCode := resp.statusCode
+	assert.Assert(t, len(receivedBytes) > 0, "json response is empty")
+	assert.Check(t, statusCode != http.StatusInternalServerError, "response status code")
+	var aggregated AggregatedStateInfo
+	err = json.Unmarshal(receivedBytes, &aggregated)
+	assert.NilError(t, err)
+	verifyStateDumpJSON(t, &aggregated)
+}
+
+func TestEnableDisablePeriodicStateDump(t *testing.T) {
+	schedulerContext = prepareSchedulerContext(t)
+	defer deleteStateDumpFile(t)
+	defer terminateGoroutine()
+
+	imHistory = history.NewInternalMetricsHistory(5)
+	req, err := http.NewRequest("PUT", "/ws/v1/periodicstatedump", strings.NewReader(""))
+	assert.NilError(t, err)
+	vars := map[string]string{
+		"periodSeconds": "3",
+		"switch":        "enable",
+	}
+	req = mux.SetURLVars(req, vars)
+	resp := &MockResponseWriter{}
+
+	// enable state dump, check file contents
+	handlePeriodicStateDump(resp, req)
+	statusCode := resp.statusCode
+	assert.Equal(t, statusCode, 0, "response status code")
+
+	waitForStateDumpFile(t)
+	fileContents, err2 := ioutil.ReadFile(stateDumpFilePath)
+	assert.NilError(t, err2)
+	var aggregated AggregatedStateInfo
+	err3 := json.Unmarshal(fileContents, &aggregated)
+	assert.NilError(t, err3)
+	verifyStateDumpJSON(t, &aggregated)
+
+	// disable
+	req, err = http.NewRequest("PUT", "/ws/v1/periodicstatedump", strings.NewReader(""))
+	assert.NilError(t, err)
+	vars = map[string]string{
+		"switch": "disable",
+	}
+	req = mux.SetURLVars(req, vars)
+	resp = &MockResponseWriter{}
+	handlePeriodicStateDump(resp, req)
+	statusCode = resp.statusCode
+	assert.Equal(t, statusCode, 0, "response status code")
+}
+
+func TestTryEnableStateDumpTwice(t *testing.T) {
+	defer deleteStateDumpFile(t)
+	defer terminateGoroutine()
+
+	req, err := http.NewRequest("PUT", "/ws/v1/periodicstatedump", strings.NewReader(""))
+	assert.NilError(t, err)
+	vars := map[string]string{
+		"switch": "enable",
+	}
+	req = mux.SetURLVars(req, vars)
+	resp := &MockResponseWriter{}
+
+	// first call - succeeds
+	handlePeriodicStateDump(resp, req)
+	statusCode := resp.statusCode
+	assert.Equal(t, statusCode, 0, "response status code")
+
+	// second call - expected to fail
+	handlePeriodicStateDump(resp, req)
+	statusCode = resp.statusCode
+	assert.Equal(t, statusCode, http.StatusInternalServerError, "response status code")
+}
+
+func TestTryDisableNotRunningStateDump(t *testing.T) {
+	req, err := http.NewRequest("PUT", "/ws/v1/periodicstatedump", strings.NewReader(""))
+	assert.NilError(t, err)
+	vars := map[string]string{
+		"switch": "disable",
+	}
+	req = mux.SetURLVars(req, vars)
+	resp := &MockResponseWriter{}
+
+	handlePeriodicStateDump(resp, req)
+
+	statusCode := resp.statusCode
+	assert.Equal(t, statusCode, http.StatusInternalServerError, "response status code")
+}
+
+func TestIllegalStateDumpRequests(t *testing.T) {
+	// missing
+	req, err := http.NewRequest("PUT", "/ws/v1/periodicstatedump", strings.NewReader(""))
+	assert.NilError(t, err)
+	resp := &MockResponseWriter{}
+	handlePeriodicStateDump(resp, req)
+	statusCode := resp.statusCode
+	assert.Equal(t, statusCode, http.StatusBadRequest, "response status code")
+
+	// illegal
+	req, err = http.NewRequest("PUT", "/ws/v1/periodicstatedump", strings.NewReader(""))
+	assert.NilError(t, err)
+	vars := map[string]string{
+		"switch": "illegal",
+	}
+	req = mux.SetURLVars(req, vars)
+	resp = &MockResponseWriter{}
+	handlePeriodicStateDump(resp, req)
+	statusCode = resp.statusCode
+	assert.Equal(t, statusCode, http.StatusBadRequest, "response status code")
+}
+
+func prepareSchedulerContext(t *testing.T) *scheduler.ClusterContext {
+	configs.MockSchedulerConfigByData([]byte(configDefault))
+	var err error
+	schedulerContext, err = scheduler.NewClusterContext(rmID, policyGroup)
+	assert.NilError(t, err, "Error when load clusterInfo from config")
+	assert.Equal(t, 1, len(schedulerContext.GetPartitionMapClone()))
+
+	return schedulerContext
+}
+
+func waitForStateDumpFile(t *testing.T) {
+	var attempts int
+	for {
+		info, err := os.Stat(stateDumpFilePath)
+		// tolerate only "file not found" errors
+		if err != nil && !os.IsNotExist(err) {
+			t.Fatal(err)
+		}
+
+		if info != nil && info.Size() > 0 {
+			break
+		}
+
+		if attempts++; attempts > 10 {
+			t.Fatal("state dump file has not been created or still empty")
+		}
+
+		time.Sleep(1 * time.Second)
+	}
+}
+
+func deleteStateDumpFile(t *testing.T) {
+	if err := os.Remove(stateDumpFilePath); err != nil {
+		t.Fatal(err)
+	}
+}
+
+func terminateGoroutine() {
+	if !isAbortClosed() {
+		abort <- struct{}{}
+		close(abort)
+		periodicStateDump = false
+	}
+}
+
+func isAbortClosed() bool {
+	select {
+	case <-abort:
+		return true
+	default:
+	}
+	return false
+}
+
+func verifyStateDumpJSON(t *testing.T, aggregated *AggregatedStateInfo) {
+	assert.Check(t, len(aggregated.Timestamp) > 0)
+	assert.Check(t, len(aggregated.Partitions) > 0)
+	assert.Check(t, len(aggregated.Nodes) > 0)
+	assert.Check(t, len(aggregated.ClusterInfo) > 0)
+	assert.Check(t, len(aggregated.ClusterUtilization) > 0)
+	assert.Check(t, len(aggregated.Queues) > 0)
+	assert.Check(t, len(aggregated.LogLevel) > 0)
+}
