@@ -23,7 +23,6 @@ import (
 
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/labels"
-	clientv1 "k8s.io/client-go/listers/core/v1"
 )
 
 type NodeListerMock struct {
@@ -52,8 +51,3 @@ func (n *NodeListerMock) Get(name string) (*v1.Node, error) {
 	}
 	return nil, fmt.Errorf("node %s is not found", name)
 }
-
-func (n *NodeListerMock) ListWithPredicate(predicate clientv1.NodeConditionPredicate) ([]*v1.Node, error) {
-	// ignore predicates for now
-	return n.allNodes, nil
-}
