@@ -29,11 +29,12 @@ import (
 	"strings"
 
 	"go.uber.org/zap"
-	"k8s.io/api/admission/v1beta1"
+	admissionv1 "k8s.io/api/admission/v1"
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/serializer"
+	"k8s.io/apimachinery/pkg/types"
 
 	"github.com/apache/incubator-yunikorn-k8shim/pkg/common/constants"
 	"github.com/apache/incubator-yunikorn-k8shim/pkg/log"
@@ -45,6 +46,9 @@ const (
 	enableConfigHotRefreshEnvVar = "ENABLE_CONFIG_HOT_REFRESH"
 	yunikornPod                  = "yunikorn"
 	defaultQueue                 = "root.default"
+	configHotFreshResponse       = "ConfigHotRefresh is disabled. Please use the REST API to update the configuration, or enable configHotRefresh. "
+	admissionReviewAPIVersion    = "admission.k8s.io/v1"
+	admissionReviewKind          = "AdmissionReview"
 )
 
 var (
@@ -69,9 +73,31 @@ type ValidateConfResponse struct {
 	Reason  string `json:"reason"`
 }
 
-func (c *admissionController) mutate(ar *v1beta1.AdmissionReview) *v1beta1.AdmissionResponse {
-	req := ar.Request
-	namespace := ar.Request.Namespace
+func admissionResponseBuilder(uid string, allowed bool, resultMessage string, patch []byte) *admissionv1.AdmissionResponse {
+	res := &admissionv1.AdmissionResponse{}
+	res.Allowed = allowed
+	res.UID = types.UID(uid)
+
+	if len(resultMessage) != 0 {
+		res.Result = &metav1.Status{
+			Message: resultMessage,
+		}
+	}
+	if len(patch) != 0 {
+		res.Patch = patch
+		pt := admissionv1.PatchTypeJSONPatch
+		res.PatchType = &pt
+	}
+	return res
+}
+
+func (c *admissionController) mutate(req *admissionv1.AdmissionRequest) *admissionv1.AdmissionResponse {
+	if req == nil {
+		log.Logger().Warn("empty request received")
+		return admissionResponseBuilder("", false, "", nil)
+	}
+
+	namespace := req.Namespace
 	var patch []patchOperation
 
 	var requestKind = req.Kind.Kind
@@ -81,9 +107,7 @@ func (c *admissionController) mutate(ar *v1beta1.AdmissionReview) *v1beta1.Admis
 		log.Logger().Warn("request kind is not pod", zap.String("uid", uid),
 			zap.String("requestKind", requestKind))
 
-		return &v1beta1.AdmissionResponse{
-			Allowed: true,
-		}
+		return admissionResponseBuilder(uid, true, "", nil)
 	}
 
 	log.Logger().Info("AdmissionReview",
@@ -95,20 +119,13 @@ func (c *admissionController) mutate(ar *v1beta1.AdmissionReview) *v1beta1.Admis
 	var pod v1.Pod
 	if err := json.Unmarshal(req.Object.Raw, &pod); err != nil {
 		log.Logger().Error("unmarshal failed", zap.Error(err))
-		return &v1beta1.AdmissionResponse{
-			Allowed: false,
-			Result: &metav1.Status{
-				Message: err.Error(),
-			},
-		}
+		return admissionResponseBuilder(uid, false, err.Error(), nil)
 	}
 
 	if labelAppValue, ok := pod.Labels[constants.LabelApp]; ok {
 		if labelAppValue == yunikornPod {
 			log.Logger().Info("ignore yunikorn pod")
-			return &v1beta1.AdmissionResponse{
-				Allowed: true,
-			}
+			return admissionResponseBuilder(uid, true, "", nil)
 		}
 	}
 
@@ -120,22 +137,10 @@ func (c *admissionController) mutate(ar *v1beta1.AdmissionReview) *v1beta1.Admis
 	patchBytes, err := json.Marshal(patch)
 	if err != nil {
 		log.Logger().Error("failed to marshal patch", zap.Error(err))
-		return &v1beta1.AdmissionResponse{
-			Allowed: false,
-			Result: &metav1.Status{
-				Message: err.Error(),
-			},
-		}
+		return admissionResponseBuilder(uid, false, err.Error(), nil)
 	}
 
-	return &v1beta1.AdmissionResponse{
-		Allowed: true,
-		Patch:   patchBytes,
-		PatchType: func() *v1beta1.PatchType {
-			pt := v1beta1.PatchTypeJSONPatch
-			return &pt
-		}(),
-	}
+	return admissionResponseBuilder(uid, true, "", patchBytes)
 }
 
 func updateSchedulerName(patch []patchOperation) []patchOperation {
@@ -213,51 +218,36 @@ func isConfigMapUpdateAllowed(userInfo string) bool {
 	return false
 }
 
-func (c *admissionController) validateConf(ar *v1beta1.AdmissionReview) *v1beta1.AdmissionResponse {
-	req := ar.Request
+func (c *admissionController) validateConf(req *admissionv1.AdmissionRequest) *admissionv1.AdmissionResponse {
+	if req == nil {
+		log.Logger().Warn("empty request received")
+		return admissionResponseBuilder("", false, "", nil)
+	}
+
+	uid := string(req.UID)
 	if !isConfigMapUpdateAllowed(req.UserInfo.Username) {
-		return &v1beta1.AdmissionResponse{
-			Allowed: false,
-			Result: &metav1.Status{
-				Message: fmt.Sprintf("ConfigHotRefresh is disabled. " +
-					"Please use the REST API to update the configuration, or enable configHotRefresh"),
-			},
-		}
+		return admissionResponseBuilder(uid, false, configHotFreshResponse, nil)
 	}
 
 	var requestKind = req.Kind.Kind
 	if requestKind != "ConfigMap" {
 		log.Logger().Warn("request kind is not configmap", zap.String("requestKind", requestKind))
-		return &v1beta1.AdmissionResponse{
-			Allowed: true,
-		}
+		return admissionResponseBuilder(uid, true, "", nil)
 	}
 
 	var configmap v1.ConfigMap
 	if err := json.Unmarshal(req.Object.Raw, &configmap); err != nil {
 		log.Logger().Error("failed to unmarshal configmap", zap.Error(err))
-		return &v1beta1.AdmissionResponse{
-			Allowed: false,
-			Result: &metav1.Status{
-				Message: err.Error(),
-			},
-		}
+		return admissionResponseBuilder(uid, false, err.Error(), nil)
 	}
 
 	// validate new/updated config map
 	if err := c.validateConfigMap(&configmap); err != nil {
 		log.Logger().Error("failed to validate yunikorn configs", zap.Error(err))
-		return &v1beta1.AdmissionResponse{
-			Allowed: false,
-			Result: &metav1.Status{
-				Message: err.Error(),
-			},
-		}
+		return admissionResponseBuilder(uid, false, err.Error(), nil)
 	}
 
-	return &v1beta1.AdmissionResponse{
-		Allowed: true,
-	}
+	return admissionResponseBuilder(uid, true, "", nil)
 }
 
 func (c *admissionController) validateConfigMap(cm *v1.ConfigMap) error {
@@ -289,55 +279,73 @@ func (c *admissionController) validateConfigMap(cm *v1.ConfigMap) error {
 	return nil
 }
 
+func (c *admissionController) health(w http.ResponseWriter, r *http.Request) {
+	// for now, always healthy
+	w.Header().Set("Content-type", "text/plain")
+	w.WriteHeader(200)
+	_, err := w.Write([]byte("OK\r\n"))
+	if err != nil {
+		log.Logger().Error("Unable to write health check result", zap.Error(err))
+		return
+	}
+}
+
 func (c *admissionController) serve(w http.ResponseWriter, r *http.Request) {
 	log.Logger().Debug("request", zap.Any("httpRequest", r))
 	var body []byte
 	if r.Body != nil {
-		if data, err := ioutil.ReadAll(r.Body); err == nil {
-			body = data
+		var err error
+		body, err = ioutil.ReadAll(r.Body)
+		if err != nil || len(body) == 0 {
+			log.Logger().Debug("illegal request received: body invalid", zap.Error(err))
+			http.Error(w, "empty or invalid body", http.StatusBadRequest)
+			return
 		}
 	}
-	if len(body) == 0 {
-		http.Error(w, "empty body", http.StatusBadRequest)
-		return
-	}
 
 	// verify the content type is accurate
 	contentType := r.Header.Get("Content-Type")
 	if contentType != "application/json" {
+		log.Logger().Debug("illegal request received: invalid content type", zap.String("requested content type", contentType))
 		http.Error(w, "invalid Content-Type, expect `application/json`", http.StatusUnsupportedMediaType)
 		return
 	}
 
-	var admissionResponse *v1beta1.AdmissionResponse
-	ar := v1beta1.AdmissionReview{}
 	urlPath := r.URL.Path
+	if urlPath != mutateURL && urlPath != validateConfURL {
+		log.Logger().Debug("unsupported request received", zap.String("urlPath", urlPath))
+		http.Error(w, "request is neither mutation nor validation", http.StatusNotFound)
+		return
+	}
+	ar := admissionv1.AdmissionReview{TypeMeta: metav1.TypeMeta{
+		APIVersion: admissionReviewAPIVersion,
+		Kind:       admissionReviewKind,
+	}}
 
-	if _, _, err := deserializer.Decode(body, nil, &ar); err != nil {
-		log.Logger().Error("Can't decode the body", zap.Error(err))
-		admissionResponse = &v1beta1.AdmissionResponse{
-			Allowed: false,
-			Result: &metav1.Status{
-				Message: err.Error(),
-			},
-		}
-	} else if urlPath == mutateURL {
-		admissionResponse = c.mutate(&ar)
-	} else if urlPath == validateConfURL {
-		admissionResponse = c.validateConf(&ar)
+	var admissionResponse *admissionv1.AdmissionResponse
+	_, _, err := deserializer.Decode(body, nil, &ar)
+	if err != nil || ar.Request == nil {
+		log.Logger().Error("request body decode failed or request empty", zap.Error(err))
+		admissionResponse = admissionResponseBuilder("yunikorn-invalid-body", false, "body decode failed", nil)
 	} else {
-		log.Logger().Warn("request is neither mutation nor validation", zap.String("urlPath", urlPath))
-	}
-
-	admissionReview := v1beta1.AdmissionReview{}
-	if admissionResponse != nil {
-		admissionReview.Response = admissionResponse
-		if ar.Request != nil {
-			admissionReview.Response.UID = ar.Request.UID
+		req := ar.Request
+		switch urlPath {
+		case mutateURL:
+			admissionResponse = c.mutate(req)
+		case validateConfURL:
+			admissionResponse = c.validateConf(req)
 		}
 	}
+	admissionReview := admissionv1.AdmissionReview{
+		TypeMeta: metav1.TypeMeta{
+			APIVersion: admissionReviewAPIVersion,
+			Kind:       admissionReviewKind,
+		},
+		Response: admissionResponse,
+	}
 
-	resp, err := json.Marshal(admissionReview)
+	var resp []byte
+	resp, err = json.Marshal(admissionReview)
 	if err != nil {
 		errMessage := fmt.Sprintf("could not encode response: %v", err)
 		log.Logger().Error(errMessage)
